@@ -1,18 +1,18 @@
-/* 
- * Copyright (c) 2007 Secure Endpoints Inc.
+/*
+ * Copyright (c) 2007-2010 Secure Endpoints Inc.
  *
  * All rights reserved.
- * 
- * Redistribution and use in source and binary forms, with or without 
- * modification, are permitted provided that the following conditions 
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
  * are met:
- * 
- *     * Redistributions of source code must retain the above copyright 
+ *
+ *     * Redistributions of source code must retain the above copyright
  *       notice, this list of conditions and the following disclaimer.
- *     * Neither the name of the Secure Endpoints Inc. nor the names of its 
- *       contributors may be used to endorse or promote products derived 
+ *     * Neither the name of the Secure Endpoints Inc. nor the names of its
+ *       contributors may be used to endorse or promote products derived
  *       from this software without specific prior written permission.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -26,12 +26,15 @@
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-/* This source file provides the declarations 
+/* This source file provides the declarations
  * which specify the AFS Cache Manager Volume Status Event
  * Notification API
  */
 
+#include <afsconfig.h>
 #include <afs/param.h>
+#include <roken.h>
+
 #include <afs/stds.h>
 
 #include <windows.h>
@@ -43,22 +46,41 @@
 #include "smb.h"
 #include <WINNT/afsreg.h>
 
+extern DWORD RDR_NetworkAddrChange(void);
+extern DWORD RDR_VolumeStatus(ULONG cellID, ULONG volID, BOOLEAN online);
+extern DWORD RDR_NetworkStatus(BOOLEAN status);
+
 HMODULE hVolStatus = NULL;
 dll_VolStatus_Funcs_t dll_funcs;
 cm_VolStatus_Funcs_t cm_funcs;
 
 static char volstat_NetbiosName[64] = "";
 
+static DWORD RDR_Notifications = 0;
+
+rdr_volstat_evt_t *rdr_evtH = NULL;
+rdr_volstat_evt_t *rdr_evtT = NULL;
+
+static EVENT_HANDLE rdr_q_event = NULL;
+
+static osi_mutex_t rdr_evt_lock;
+
+void
+cm_VolStatus_SetRDRNotifications(DWORD onoff)
+{
+    RDR_Notifications = onoff;
+}
+
 afs_uint32
 cm_VolStatus_Active(void)
 {
     return (hVolStatus != NULL);
 }
 
-/* This function is used to load any Volume Status Handlers 
- * and their associated function pointers.  
+/* This function is used to load any Volume Status Handlers
+ * and their associated function pointers.
  */
-long 
+long
 cm_VolStatus_Initialization(void)
 {
     long (__fastcall * dll_VolStatus_Initialization)(dll_VolStatus_Funcs_t * dll_funcs, cm_VolStatus_Funcs_t *cm_funcs) = NULL;
@@ -74,16 +96,21 @@ cm_VolStatus_Initialization(void)
         code = RegQueryValueEx(parmKey, "VolStatusHandler", NULL, NULL,
                                 (BYTE *) &wd, &dummyLen);
 
-        if (code == 0) {
+        if (code == ERROR_SUCCESS) {
             dummyLen = sizeof(volstat_NetbiosName);
-            code = RegQueryValueEx(parmKey, "NetbiosName", NULL, NULL, 
+            code = RegQueryValueEx(parmKey, "NetbiosName", NULL, NULL,
                                    (BYTE *)volstat_NetbiosName, &dummyLen);
         }
+        if (code == ERROR_SUCCESS && wd[0])
+            hVolStatus = LoadLibrary(wd);
+
+        dummyLen = sizeof(wd);
+        code = RegQueryValueEx(parmKey, "RDRVolStatNotify", NULL, NULL,
+                                (BYTE *) &RDR_Notifications, &dummyLen);
+
         RegCloseKey (parmKey);
     }
 
-    if (code == ERROR_SUCCESS && wd[0])
-        hVolStatus = LoadLibrary(wd);
     if (hVolStatus) {
         (FARPROC) dll_VolStatus_Initialization = GetProcAddress(hVolStatus, "@VolStatus_Initialization@8");
         if (dll_VolStatus_Initialization) {
@@ -93,9 +120,9 @@ cm_VolStatus_Initialization(void)
 
             dll_funcs.version = DLL_VOLSTATUS_FUNCS_VERSION;
             code = dll_VolStatus_Initialization(&dll_funcs, &cm_funcs);
-        } 
+        }
 
-        if (dll_VolStatus_Initialization == NULL || code != 0 || 
+        if (dll_VolStatus_Initialization == NULL || code != 0 ||
             dll_funcs.version != DLL_VOLSTATUS_FUNCS_VERSION) {
             FreeLibrary(hVolStatus);
             hVolStatus = NULL;
@@ -103,6 +130,23 @@ cm_VolStatus_Initialization(void)
         }
     }
 
+    if (RDR_Initialized && RDR_Notifications) {
+        long pid;
+        thread_t phandle;
+
+        lock_InitializeMutex(&rdr_evt_lock, "rdr_evt_lock", LOCK_HIERARCHY_IGNORE);
+
+        phandle = thrd_Create((SecurityAttrib) NULL, 0,
+                                       (ThreadFunc) cm_VolStatus_DeliverNotifications,
+                                       0, 0, &pid, "cm_VolStatus_DeliverNotifications");
+        osi_assertx(phandle != NULL, "cm_VolStatus_DeliverNotifications thread creation failure");
+        thrd_CloseHandle(phandle);
+
+        rdr_q_event = thrd_CreateEvent(NULL, TRUE, TRUE, "rdr_q_event");
+        if ( GetLastError() == ERROR_ALREADY_EXISTS )
+            afsi_log("Event Object Already Exists: rdr_q_event");
+    }
+
     osi_Log1(afsd_logp,"cm_VolStatus_Initialization 0x%x", code);
 
     return code;
@@ -111,11 +155,15 @@ cm_VolStatus_Initialization(void)
 /* This function is used to unload any Volume Status Handlers
  * that were loaded during initialization.
  */
-long 
+long
 cm_VolStatus_Finalize(void)
 {
     osi_Log1(afsd_logp,"cm_VolStatus_Finalize handle 0x%x", hVolStatus);
 
+    if ( RDR_Initialized && RDR_Notifications ) {
+        CloseHandle(rdr_q_event);
+    }
+
     if (hVolStatus == NULL)
         return 0;
 
@@ -128,7 +176,7 @@ cm_VolStatus_Finalize(void)
  * AFS client service has started.  If the network is started
  * at this point we call cm_VolStatus_Network_Started().
  */
-long 
+long
 cm_VolStatus_Service_Started(void)
 {
     long code = 0;
@@ -137,7 +185,7 @@ cm_VolStatus_Service_Started(void)
 
     if (hVolStatus == NULL)
         return 0;
-   
+
     code = dll_funcs.dll_VolStatus_Service_Started();
     if (code == 0 && smb_IsNetworkStarted())
         code = dll_funcs.dll_VolStatus_Network_Started(cm_NetbiosName, cm_NetbiosName);
@@ -148,7 +196,7 @@ cm_VolStatus_Service_Started(void)
 /* This function notifies the Volume Status Handlers that the
  * AFS client service is stopping.
  */
-long 
+long
 cm_VolStatus_Service_Stopped(void)
 {
     long code = 0;
@@ -157,7 +205,7 @@ cm_VolStatus_Service_Stopped(void)
 
     if (hVolStatus == NULL)
         return 0;
-   
+
     code = dll_funcs.dll_VolStatus_Service_Stopped();
 
     return code;
@@ -165,7 +213,7 @@ cm_VolStatus_Service_Stopped(void)
 
 
 /* This function notifies the Volume Status Handlers that the
- * AFS client service is accepting network requests using the 
+ * AFS client service is accepting network requests using the
  * specified netbios names.
  */
 long
@@ -177,6 +225,18 @@ cm_VolStatus_Network_Started(const char * netbios32)
 {
     long code = 0;
 
+    if (RDR_Initialized && RDR_Notifications) {
+        rdr_volstat_evt_t *evp = (rdr_volstat_evt_t *)malloc(sizeof(rdr_volstat_evt_t));
+        evp->type = netstatus;
+        evp->netstatus_data.status = TRUE;
+
+        lock_ObtainMutex(&rdr_evt_lock);
+        osi_QAddH((osi_queue_t **) &rdr_evtH, (osi_queue_t **) &rdr_evtT, &evp->q);
+        lock_ReleaseMutex(&rdr_evt_lock);
+
+        thrd_SetEvent(rdr_q_event);
+    }
+
     if (hVolStatus == NULL)
         return 0;
 
@@ -190,8 +250,8 @@ cm_VolStatus_Network_Started(const char * netbios32)
 }
 
 /* This function notifies the Volume Status Handlers that the
- * AFS client service is no longer accepting network requests 
- * using the specified netbios names 
+ * AFS client service is no longer accepting network requests
+ * using the specified netbios names
  */
 long
 #ifdef _WIN64
@@ -202,6 +262,18 @@ cm_VolStatus_Network_Stopped(const char * netbios32)
 {
     long code = 0;
 
+    if (RDR_Initialized && RDR_Notifications) {
+        rdr_volstat_evt_t *evp = (rdr_volstat_evt_t *)malloc(sizeof(rdr_volstat_evt_t));
+        evp->type = netstatus;
+        evp->netstatus_data.status = FALSE;
+
+        lock_ObtainMutex(&rdr_evt_lock);
+        osi_QAddH((osi_queue_t **) &rdr_evtH, (osi_queue_t **) &rdr_evtT, &evp->q);
+        lock_ReleaseMutex(&rdr_evt_lock);
+
+        thrd_SetEvent(rdr_q_event);
+    }
+
     if (hVolStatus == NULL)
         return 0;
 
@@ -215,15 +287,26 @@ cm_VolStatus_Network_Stopped(const char * netbios32)
 }
 
 /* This function is called when the IP address list changes.
- * Volume Status Handlers can use this notification as a hint 
- * that it might be possible to determine volume IDs for paths 
- * that previously were not accessible.  
+ * Volume Status Handlers can use this notification as a hint
+ * that it might be possible to determine volume IDs for paths
+ * that previously were not accessible.
  */
-long 
+long
 cm_VolStatus_Network_Addr_Change(void)
 {
     long code = 0;
 
+    if (RDR_Initialized && RDR_Notifications) {
+        rdr_volstat_evt_t *evp = (rdr_volstat_evt_t *)malloc(sizeof(rdr_volstat_evt_t));
+        evp->type = addrchg;
+
+        lock_ObtainMutex(&rdr_evt_lock);
+        osi_QAddH((osi_queue_t **) &rdr_evtH, (osi_queue_t **) &rdr_evtT, &evp->q);
+        lock_ReleaseMutex(&rdr_evt_lock);
+
+        thrd_SetEvent(rdr_q_event);
+    }
+
     if (hVolStatus == NULL)
         return 0;
 
@@ -232,14 +315,42 @@ cm_VolStatus_Network_Addr_Change(void)
     return code;
 }
 
-/* This function notifies the Volume Status Handlers that the 
+/* This function notifies the Volume Status Handlers that the
  * state of the specified cell.volume has changed.
  */
-long 
+long
 cm_VolStatus_Change_Notification(afs_uint32 cellID, afs_uint32 volID, enum volstatus status)
 {
     long code = 0;
 
+    if (RDR_Initialized && RDR_Notifications) {
+        rdr_volstat_evt_t *evp = (rdr_volstat_evt_t *)malloc(sizeof(rdr_volstat_evt_t));
+        switch (status) {
+        case vl_alldown:
+        case vl_offline:
+            evp->type = volstatus;
+            evp->volstatus_data.cellID = cellID;
+            evp->volstatus_data.volID = volID;
+            evp->volstatus_data.online = FALSE;
+
+            lock_ObtainMutex(&rdr_evt_lock);
+            osi_QAddH((osi_queue_t **) &rdr_evtH, (osi_queue_t **) &rdr_evtT, &evp->q);
+            lock_ReleaseMutex(&rdr_evt_lock);
+            break;
+        default:
+            evp->type = volstatus;
+            evp->volstatus_data.cellID = cellID;
+            evp->volstatus_data.volID = volID;
+            evp->volstatus_data.online = TRUE;
+
+            lock_ObtainMutex(&rdr_evt_lock);
+            osi_QAddH((osi_queue_t **) &rdr_evtH, (osi_queue_t **) &rdr_evtT, &evp->q);
+            lock_ReleaseMutex(&rdr_evt_lock);
+        }
+
+        thrd_SetEvent(rdr_q_event);
+    }
+
     if (hVolStatus == NULL)
         return 0;
 
@@ -317,7 +428,7 @@ cm_VolStatus_Path_To_ID(const char * share, const char * path, afs_uint32 * cell
     if (cellID == NULL || volID == NULL)
         return CM_ERROR_INVAL;
 
-    osi_Log2(afsd_logp,"cm_VolStatus_Path_To_ID share %s path %s", 
+    osi_Log2(afsd_logp,"cm_VolStatus_Path_To_ID share %s path %s",
               osi_LogSaveString(afsd_logp, (char *)share), osi_LogSaveString(afsd_logp, (char *)path));
 
     cm_InitReq(&req);
@@ -325,7 +436,14 @@ cm_VolStatus_Path_To_ID(const char * share, const char * path, afs_uint32 * cell
     cpath = cm_FsStringToClientStringAlloc(path, -1, NULL);
     cshare = cm_FsStringToClientStringAlloc(share, -1, NULL);
 
-    code = cm_NameI(cm_data.rootSCachep, cpath,
+    if (cpath == NULL || cshare == NULL) {
+        osi_Log1(afsd_logp, "Can't convert %s string. Aborting",
+                 (cpath == NULL)? "path" : "share");
+        code = CM_ERROR_NOSUCHPATH;
+        goto done;
+    }
+
+    code = cm_NameI(cm_RootSCachep(cm_rootUserp, &req), cpath,
                     CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
                     cm_rootUserp, cshare, &req, &scp);
     if (code)
@@ -339,7 +457,7 @@ cm_VolStatus_Path_To_ID(const char * share, const char * path, afs_uint32 * cell
         cm_ReleaseSCache(scp);
         goto done;
     }
-        
+
     cm_SyncOpDone(scp, NULL, CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS);
 
     *cellID = scp->fid.cell;
@@ -360,7 +478,7 @@ cm_VolStatus_Path_To_ID(const char * share, const char * path, afs_uint32 * cell
     if (cshare)
         free(cshare);
 
-    osi_Log1(afsd_logp,"cm_VolStatus_Path_To_ID code 0x%x",code); 
+    osi_Log1(afsd_logp,"cm_VolStatus_Path_To_ID code 0x%x",code);
     return code;
 }
 
@@ -377,7 +495,7 @@ cm_VolStatus_Path_To_DFSlink(const char * share, const char * path, afs_uint32 *
     if (pBufSize == NULL || (pBuffer == NULL && *pBufSize != 0))
         return CM_ERROR_INVAL;
 
-    osi_Log2(afsd_logp,"cm_VolStatus_Path_To_DFSlink share %s path %s", 
+    osi_Log2(afsd_logp,"cm_VolStatus_Path_To_DFSlink share %s path %s",
               osi_LogSaveString(afsd_logp, (char *)share), osi_LogSaveString(afsd_logp, (char *)path));
 
     cm_InitReq(&req);
@@ -385,7 +503,14 @@ cm_VolStatus_Path_To_DFSlink(const char * share, const char * path, afs_uint32 *
     cpath = cm_FsStringToClientStringAlloc(path, -1, NULL);
     cshare = cm_FsStringToClientStringAlloc(share, -1, NULL);
 
-    code = cm_NameI(cm_data.rootSCachep, cpath, CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW, 
+    if (cpath == NULL || cshare == NULL) {
+        osi_Log1(afsd_logp, "Can't convert %s string. Aborting",
+                 (cpath == NULL)? "path" : "share");
+        code = CM_ERROR_NOSUCHPATH;
+        goto done;
+    }
+
+    code = cm_NameI(cm_RootSCachep(cm_rootUserp, &req), cpath, CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
                     cm_rootUserp, cshare, &req, &scp);
     if (code)
         goto done;
@@ -398,7 +523,7 @@ cm_VolStatus_Path_To_DFSlink(const char * share, const char * path, afs_uint32 *
         cm_ReleaseSCache(scp);
         goto done;
     }
-        
+
     cm_SyncOpDone(scp, NULL, CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS);
 
     if (scp->fileType != CM_SCACHETYPE_DFSLINK) {
@@ -426,6 +551,41 @@ cm_VolStatus_Path_To_DFSlink(const char * share, const char * path, afs_uint32 *
     if (cshare)
         free(cshare);
 
-    osi_Log1(afsd_logp,"cm_VolStatus_Path_To_DFSlink code 0x%x",code); 
+    osi_Log1(afsd_logp,"cm_VolStatus_Path_To_DFSlink code 0x%x",code);
     return code;
 }
+
+void
+cm_VolStatus_DeliverNotifications(void * dummy)
+{
+    rdr_volstat_evt_t *evp, *evprev;
+    afs_uint32 code;
+
+    while ( TRUE ) {
+        code = thrd_WaitForSingleObject_Event( rdr_q_event, INFINITE );
+
+        lock_ObtainMutex(&rdr_evt_lock);
+        for (evp = rdr_evtT; evp; evp = evprev)
+        {
+            evprev = (rdr_volstat_evt_t *) osi_QPrev(&evp->q);
+            osi_QRemoveHT((osi_queue_t **) &rdr_evtH, (osi_queue_t **) &rdr_evtT, &evp->q);
+            lock_ReleaseMutex(&rdr_evt_lock);
+
+            switch ( evp->type ) {
+            case addrchg:
+                RDR_NetworkAddrChange();
+                break;
+            case volstatus:
+                RDR_VolumeStatus(evp->volstatus_data.cellID, evp->volstatus_data.volID, evp->volstatus_data.online);
+                break;
+            case netstatus:
+                RDR_NetworkStatus(evp->netstatus_data.status);
+                break;
+            }
+
+            free(evp);
+            lock_ObtainMutex(&rdr_evt_lock);
+        }
+        lock_ReleaseMutex(&rdr_evt_lock);
+    }
+}
