@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -33,6 +33,7 @@ typedef unsigned short etap_event_t;
 #include <sys/lock.h>
 #include <sys/user.h>
 #include <sys/vnode.h>
+#include <sys/kauth.h>
 #endif
 #include <kern/thread.h>
 
@@ -97,14 +98,14 @@ enum vcexcl { EXCL, NONEXCL };
 #define va_atime va_access_time
 #define va_mtime va_modify_time
 #define va_ctime va_change_time
-#define va_bytes va_total_alloc 
+#define va_bytes va_total_alloc
 #define va_blocksize va_iosize
 #define va_nodeid va_fileid
 
 #define crref kauth_cred_get_with_ref
 #define crhold kauth_cred_ref
 #ifdef AFS_DARWIN100_ENV
-#define crfree(X) kauth_cred_unref(&X)
+static inline void crfree(kauth_cred_t X) { kauth_cred_unref(&X); }
 #else
 #define crfree kauth_cred_rele
 #endif
@@ -120,7 +121,7 @@ extern int afs_osi_ctxtp_initialized;
 extern u_int32_t afs_darwin_realmodes;
 extern u_int32_t afs_darwin_fsevents;
 
-/* 
+/*
  * Time related macros
  */
 #ifdef AFS_DARWIN80_ENV
@@ -145,9 +146,6 @@ typedef struct proc afs_proc_t;
 #define VN_RELE(vp) vrele(vp);
 
 void darwin_vn_hold(struct vnode *vp);
-#ifdef AFS_DARWIN80_ENV
-void darwin_vn_rele(struct vnode *vp);
-#endif
 
 #define gop_rdwr(rw,gp,base,len,offset,segflg,unit,cred,aresid) \
   vn_rdwr((rw),(gp),(base),(len),(offset),(segflg),(unit),(cred),(aresid),current_proc())
@@ -225,12 +223,12 @@ extern int igetinode(mount_t vfsp, dev_t dev , ino_t inode, vnode_t *vpp,
 #define osi_curproc() current_proc()
 
 /* FIXME */
-#define osi_curcred() &afs_osi_cred 
+#define osi_curcred() &afs_osi_cred
 
 #ifdef AFS_DARWIN80_ENV
-uio_t afsio_darwin_partialcopy(uio_t auio, int size);
-
-#define uprintf printf
+# define afsio_free(X) uio_free(X)
+# define afsio_setoffset(X, Y) uio_setoffset(X, Y)
+# define uprintf printf
 #endif
 
 /* Vnode related macros */
@@ -251,4 +249,10 @@ extern int (**afs_vnodeop_p) ();
 # define SetAfsVnode(v)     /* nothing; done in getnewvnode() */
 #endif
 
+#ifdef AFS_DARWIN80_ENV
+#define osi_procname(procname, size) proc_selfname(procname, size)
+#else
+#define osi_procname(procname, size) strncpy(procname, curproc->p_comm, size)
+#endif
+
 #endif /* _OSI_MACHDEP_H_ */
