@@ -18,7 +18,6 @@
 
 
 int afs_osicred_initialized = 0;
-AFS_UCRED afs_osi_cred;
 afs_lock_t afs_xosi;		/* lock is for tvattr */
 extern struct osi_dev cacheDev;
 extern struct vfs *afs_cacheVfsp;
@@ -43,11 +42,7 @@ afs_InitDualFSCacheOps(struct vnode *vp)
     int code;
     static int inited = 0;
     struct vfs *vfsp;
-#ifdef AFS_SUN56_ENV
     struct statvfs64 vfst;
-#else /* AFS_SUN56_ENV */
-    struct statvfs vfst;
-#endif /* AFS_SUN56_ENV */
 
     if (inited)
 	return;
@@ -85,9 +80,9 @@ VnodeToIno(vnode_t * vp)
 
     vattr.va_mask = AT_FSID | AT_NODEID;	/* quick return using this mask. */
 #ifdef AFS_SUN511_ENV
-    code = VOP_GETATTR(vp, &vattr, 0, &afs_osi_cred, NULL);
+    code = VOP_GETATTR(vp, &vattr, 0, afs_osi_credp, NULL);
 #else
-    code = VOP_GETATTR(vp, &vattr, 0, &afs_osi_cred);
+    code = VOP_GETATTR(vp, &vattr, 0, afs_osi_credp);
 #endif
     if (code) {
 	osi_Panic("VnodeToIno");
@@ -104,9 +99,9 @@ VnodeToDev(vnode_t * vp)
     vattr.va_mask = AT_FSID | AT_NODEID;	/* quick return using this mask. */
     AFS_GUNLOCK();
 #ifdef AFS_SUN511_ENV
-    code = VOP_GETATTR(vp, &vattr, 0, &afs_osi_cred, NULL);
+    code = VOP_GETATTR(vp, &vattr, 0, afs_osi_credp, NULL);
 #else
-    code = VOP_GETATTR(vp, &vattr, 0, &afs_osi_cred);
+    code = VOP_GETATTR(vp, &vattr, 0, afs_osi_credp);
 #endif
     AFS_GLOCK();
     if (code) {
@@ -125,19 +120,19 @@ VnodeToSize(vnode_t * vp)
      * We lock xosi in osi_Stat, so we probably should
      * lock it here too - RWH.
      */
-    MObtainWriteLock(&afs_xosi, 578);
+    ObtainWriteLock(&afs_xosi, 578);
     vattr.va_mask = AT_SIZE;
     AFS_GUNLOCK();
 #ifdef AFS_SUN511_ENV
-    code = VOP_GETATTR(vp, &vattr, 0, &afs_osi_cred, NULL);
+    code = VOP_GETATTR(vp, &vattr, 0, afs_osi_credp, NULL);
 #else
-    code = VOP_GETATTR(vp, &vattr, 0, &afs_osi_cred);
+    code = VOP_GETATTR(vp, &vattr, 0, afs_osi_credp);
 #endif
     AFS_GLOCK();
     if (code) {
 	osi_Panic("VnodeToSize");
     }
-    MReleaseWriteLock(&afs_xosi);
+    ReleaseWriteLock(&afs_xosi);
     return (afs_int32) (vattr.va_size);
 }
 
@@ -145,10 +140,10 @@ void *
 osi_VxfsOpen(afs_dcache_id_t *ainode)
 {
     struct vnode *vp;
-    register struct osi_file *afile = NULL;
+    struct osi_file *afile = NULL;
     afs_int32 code = 0;
     int dummy;
-    afile = (struct osi_file *)osi_AllocSmallSpace(sizeof(struct osi_file));
+    afile = osi_AllocSmallSpace(sizeof(struct osi_file));
     AFS_GUNLOCK();
     code = (*vxfs_vx_vp_byino) (afs_cacheVfsp, &vp, (unsigned int)ainode->ufs);
     AFS_GLOCK();
@@ -160,7 +155,6 @@ osi_VxfsOpen(afs_dcache_id_t *ainode)
     afile->size = VnodeToSize(afile->vnode);
     afile->offset = 0;
     afile->proc = (int (*)())0;
-    afile->inum = ainode->ufs;	/* for hint validity checking */
     return (void *)afile;
 }
 #endif /* AFS_HAVE_VXFS */
@@ -173,61 +167,45 @@ osi_UfsOpen(afs_dcache_id_t *ainode)
 #else
     struct inode *ip;
 #endif
-    register struct osi_file *afile = NULL;
+    struct osi_file *afile = NULL;
     afs_int32 code = 0;
     int dummy;
-    char fname[1024];
 #ifdef AFS_CACHE_VNODE_PATH
     char namebuf[1024];
     struct pathname lookpn;
 #endif
     struct osi_stat tstat;
-    afile = (struct osi_file *)osi_AllocSmallSpace(sizeof(struct osi_file));
+    afile = osi_AllocSmallSpace(sizeof(struct osi_file));
     AFS_GUNLOCK();
 
 /*
  * AFS_CACHE_VNODE_PATH can be used with any file system, including ZFS or tmpfs.
- * The ainode is not an inode number but a signed index used to generate file names. 
+ * The ainode is not an inode number but a path.
  */
 #ifdef AFS_CACHE_VNODE_PATH
-	switch (ainode->ufs) {
-	case AFS_CACHE_CELLS_INODE:
-	    snprintf(fname, 1024, "%s/%s", afs_cachebasedir, "CellItems");
-	    break;
-	case AFS_CACHE_ITEMS_INODE:
-	    snprintf(fname, 1024, "%s/%s", afs_cachebasedir, "CacheItems");
-	    break;
-	case AFS_CACHE_VOLUME_INODE:
-	    snprintf(fname, 1024, "%s/%s", afs_cachebasedir, "VolumeItems");
-	    break;
-	default:
-	    dummy = ainode->ufs / afs_numfilesperdir;
-	    snprintf(fname, 1024, "%s/D%d/V%d", afs_cachebasedir, dummy, ainode->ufs);
-    }
-		
 	/* Can not use vn_open or lookupname, they use user's CRED() 
-     * We need to run as root So must use low level lookuppnvp 
-     * assume fname starts with /
+	 * We need to run as root So must use low level lookuppnvp
+	 * assume fname starts with /
 	 */
 
-	code = pn_get_buf(fname, AFS_UIOSYS, &lookpn, namebuf, sizeof(namebuf));
+	code = pn_get_buf(ainode->ufs, AFS_UIOSYS, &lookpn, namebuf, sizeof(namebuf));
     if (code != 0) 
-        osi_Panic("UfsOpen: pn_get_buf failed %ld %s %ld", code, fname, ainode->ufs);
+        osi_Panic("UfsOpen: pn_get_buf failed %ld %s", code, ainode->ufs);
  
 	VN_HOLD(rootdir); /* released in loopuppnvp */
 	code = lookuppnvp(&lookpn, NULL, FOLLOW, NULL, &vp, 
-           rootdir, rootdir, &afs_osi_cred); 
+           rootdir, rootdir, afs_osi_credp);
     if (code != 0)  
-        osi_Panic("UfsOpen: lookuppnvp failed %ld %s %ld", code, fname, ainode->ufs);
+        osi_Panic("UfsOpen: lookuppnvp failed %ld %s", code, ainode->ufs);
 	
 #ifdef AFS_SUN511_ENV
-    code = VOP_OPEN(&vp, FREAD|FWRITE, &afs_osi_cred, NULL);
+    code = VOP_OPEN(&vp, FREAD|FWRITE, afs_osi_credp, NULL);
 #else
-    code = VOP_OPEN(&vp, FREAD|FWRITE, &afs_osi_cred);
+    code = VOP_OPEN(&vp, FREAD|FWRITE, afs_osi_credp);
 #endif
 
     if (code != 0)
-        osi_Panic("UfsOpen: VOP_OPEN failed %ld %s %ld", code, fname, ainode->ufs);
+        osi_Panic("UfsOpen: VOP_OPEN failed %ld %s", code, ainode->ufs);
 
 #else
     code =
@@ -237,19 +215,18 @@ osi_UfsOpen(afs_dcache_id_t *ainode)
     AFS_GLOCK();
     if (code) {
 	osi_FreeSmallSpace(afile);
-	osi_Panic("UfsOpen: igetinode failed %ld %s %ld", code, fname, ainode->ufs);
+	osi_Panic("UfsOpen: igetinode failed %ld %s", code, ainode->ufs);
     }
 #ifdef AFS_CACHE_VNODE_PATH
-	afile->vnode = vp;
-	code = afs_osi_Stat(afile, &tstat);
-	afile->size = tstat.size;
+    afile->vnode = vp;
+    code = afs_osi_Stat(afile, &tstat);
+    afile->size = tstat.size;
 #else
     afile->vnode = ITOV(ip);
     afile->size = VTOI(afile->vnode)->i_size;
 #endif
     afile->offset = 0;
     afile->proc = (int (*)())0;
-    afile->inum = ainode->ufs;	/* for hint validity checking */
     return (void *)afile;
 }
 
@@ -265,9 +242,7 @@ osi_UFSOpen(afs_dcache_id_t *ainode)
 	osi_Panic("UFSOpen called for non-UFS cache\n");
     }
     if (!afs_osicred_initialized) {
-	/* valid for alpha_osf, SunOS, Ultrix */
-	memset((char *)&afs_osi_cred, 0, sizeof(AFS_UCRED));
-	crhold(&afs_osi_cred);	/* don't let it evaporate, since it is static */
+	afs_osi_credp = kcred;
 	afs_osicred_initialized = 1;
     }
 #ifdef AFS_HAVE_VXFS
@@ -278,19 +253,19 @@ osi_UFSOpen(afs_dcache_id_t *ainode)
 }
 
 int
-afs_osi_Stat(register struct osi_file *afile, register struct osi_stat *astat)
+afs_osi_Stat(struct osi_file *afile, struct osi_stat *astat)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct vattr tvattr;
     AFS_STATCNT(osi_Stat);
-    MObtainWriteLock(&afs_xosi, 320);
+    ObtainWriteLock(&afs_xosi, 320);
     /* Ufs doesn't seem to care about the flags so we pass 0 for now */
     tvattr.va_mask = AT_ALL;
     AFS_GUNLOCK();
 #ifdef AFS_SUN511_ENV 
-    code = VOP_GETATTR(afile->vnode, &tvattr, 0, &afs_osi_cred, NULL);
+    code = VOP_GETATTR(afile->vnode, &tvattr, 0, afs_osi_credp, NULL);
 #else
-    code = VOP_GETATTR(afile->vnode, &tvattr, 0, &afs_osi_cred);
+    code = VOP_GETATTR(afile->vnode, &tvattr, 0, afs_osi_credp);
 #endif
     AFS_GLOCK();
     if (code == 0) {
@@ -298,12 +273,12 @@ afs_osi_Stat(register struct osi_file *afile, register struct osi_stat *astat)
 	astat->mtime = tvattr.va_mtime.tv_sec;
 	astat->atime = tvattr.va_atime.tv_sec;
     }
-    MReleaseWriteLock(&afs_xosi);
+    ReleaseWriteLock(&afs_xosi);
     return code;
 }
 
 int
-osi_UFSClose(register struct osi_file *afile)
+osi_UFSClose(struct osi_file *afile)
 {
     AFS_STATCNT(osi_Close);
     if (afile->vnode) {
@@ -315,11 +290,11 @@ osi_UFSClose(register struct osi_file *afile)
 }
 
 int
-osi_UFSTruncate(register struct osi_file *afile, afs_int32 asize)
+osi_UFSTruncate(struct osi_file *afile, afs_int32 asize)
 {
-    AFS_UCRED *oldCred;
+    afs_ucred_t *oldCred;
     struct vattr tvattr;
-    register afs_int32 code;
+    afs_int32 code;
     struct osi_stat tstat;
     AFS_STATCNT(osi_Truncate);
 
@@ -330,7 +305,7 @@ osi_UFSTruncate(register struct osi_file *afile, afs_int32 asize)
     code = afs_osi_Stat(afile, &tstat);
     if (code || tstat.size <= asize)
 	return code;
-    MObtainWriteLock(&afs_xosi, 321);
+    ObtainWriteLock(&afs_xosi, 321);
     tvattr.va_mask = AT_SIZE;
     tvattr.va_size = asize;
     /*
@@ -341,13 +316,13 @@ osi_UFSTruncate(register struct osi_file *afile, afs_int32 asize)
     {
 	caller_context_t ct;
 
-	code = VOP_SETATTR(afile->vnode, &tvattr, 0, &afs_osi_cred, &ct);
+	code = VOP_SETATTR(afile->vnode, &tvattr, 0, afs_osi_credp, &ct);
     }
 #else
-    code = VOP_SETATTR(afile->vnode, &tvattr, 0, &afs_osi_cred);
+    code = VOP_SETATTR(afile->vnode, &tvattr, 0, afs_osi_credp);
 #endif
     AFS_GLOCK();
-    MReleaseWriteLock(&afs_xosi);
+    ReleaseWriteLock(&afs_xosi);
     return code;
 }
 
@@ -369,17 +344,13 @@ osi_DisableAtimes(struct vnode *avp)
 
 /* Generic read interface */
 int
-afs_osi_Read(register struct osi_file *afile, int offset, void *aptr,
+afs_osi_Read(struct osi_file *afile, int offset, void *aptr,
 	     afs_int32 asize)
 {
-    AFS_UCRED *oldCred;
-#if defined(AFS_SUN57_ENV)
+    afs_ucred_t *oldCred;
     ssize_t resid;
-#else
-    int resid;
-#endif
-    register afs_int32 code;
-    register afs_int32 cnt1 = 0;
+    afs_int32 code;
+    afs_int32 cnt1 = 0;
     AFS_STATCNT(osi_Read);
 
     /**
@@ -390,7 +361,7 @@ afs_osi_Read(register struct osi_file *afile, int offset, void *aptr,
 	if (!afs_shuttingdown)
 	    osi_Panic("osi_Read called with null param");
 	else
-	    return EIO;
+	    return -EIO;
     }
 
     if (offset != -1)
@@ -398,7 +369,7 @@ afs_osi_Read(register struct osi_file *afile, int offset, void *aptr,
     AFS_GUNLOCK();
     code =
 	gop_rdwr(UIO_READ, afile->vnode, (caddr_t) aptr, asize, afile->offset,
-		 AFS_UIOSYS, 0, 0, &afs_osi_cred, &resid);
+		 AFS_UIOSYS, 0, 0, afs_osi_credp, &resid);
     AFS_GLOCK();
     if (code == 0) {
 	code = asize - resid;
@@ -407,23 +378,21 @@ afs_osi_Read(register struct osi_file *afile, int offset, void *aptr,
     } else {
 	afs_Trace2(afs_iclSetp, CM_TRACE_READFAILED, ICL_TYPE_INT32, resid,
 		   ICL_TYPE_INT32, code);
-	code = -1;
+	if (code > 0) {
+	    code = -code;
+	}
     }
     return code;
 }
 
 /* Generic write interface */
 int
-afs_osi_Write(register struct osi_file *afile, afs_int32 offset, void *aptr,
+afs_osi_Write(struct osi_file *afile, afs_int32 offset, void *aptr,
 	      afs_int32 asize)
 {
-    AFS_UCRED *oldCred;
-#if defined(AFS_SUN57_ENV)
+    afs_ucred_t *oldCred;
     ssize_t resid;
-#else
-    int resid;
-#endif
-    register afs_int32 code;
+    afs_int32 code;
     AFS_STATCNT(osi_Write);
     if (!afile)
 	osi_Panic("afs_osi_Write called with null param");
@@ -432,14 +401,16 @@ afs_osi_Write(register struct osi_file *afile, afs_int32 offset, void *aptr,
     AFS_GUNLOCK();
     code =
 	gop_rdwr(UIO_WRITE, afile->vnode, (caddr_t) aptr, asize,
-		 afile->offset, AFS_UIOSYS, 0, RLIM64_INFINITY, &afs_osi_cred,
+		 afile->offset, AFS_UIOSYS, 0, RLIM64_INFINITY, afs_osi_credp,
 		 &resid);
     AFS_GLOCK();
     if (code == 0) {
 	code = asize - resid;
 	afile->offset += code;
     } else {
-	code = -1;
+	if (code > 0) {
+	    code = -code;
+	}
     }
     if (afile->proc) {
 	(*afile->proc) (afile, code);
@@ -452,7 +423,7 @@ afs_osi_Write(register struct osi_file *afile, afs_int32 offset, void *aptr,
     This routine written from the RT NFS port strategy routine.
     It has been generalized a bit, but should still be pretty clear. */
 int
-afs_osi_MapStrategy(int (*aproc) (), register struct buf *bp)
+afs_osi_MapStrategy(int (*aproc) (), struct buf *bp)
 {
     afs_int32 returnCode;
 
