@@ -21,7 +21,6 @@
 #include <afs/afsutil.h>
 #include <afs/cellconfig.h>
 #include <rx/rx.h>
-#include <sys/stat.h>
 #include <rx/xdr.h>
 #include <afs/auth.h>
 #include <afs/cellconfig.h>
@@ -37,6 +36,7 @@ static int DoStat(char *aname, struct rx_connection *aconn,
 		  int aint32p, int firstTime);
 
 #include "bosint.h"
+#include "bnode_internal.h"
 #include "bosprototypes.h"
 
 /* command offsets for bos salvage command */
@@ -888,9 +888,10 @@ ListKeys(struct cmd_syndesc *as, void *arock)
 	    break;
 	everWorked = 1;
 	/* first check if key is returned */
-	if ((!ka_KeyIsZero(ktc_to_charptr(&tkey), sizeof(tkey))) && (as->parms[1].items)) {
+	if ((!ka_KeyIsZero((char *)&tkey, sizeof(tkey)))
+	    && (as->parms[1].items)) {
 	    printf("key %d is '", kvno);
-	    ka_PrintBytes(ktc_to_charptr(&tkey), sizeof(tkey));
+	    ka_PrintBytes((char *)&tkey, sizeof(tkey));
 	    printf("'\n");
 	} else {
 	    if (keyInfo.keyCheckSum == 0)	/* shouldn't happen */
@@ -1144,6 +1145,7 @@ DoSalvage(struct rx_connection * aconn, char * aparm1, char * aparm2,
     char pbuffer[PARMBUFFERSSIZE];
     afs_int32 partNumber;
     char *notifier = NONOTIFIER;
+    int count;
 
     /* if a partition was specified, canonicalize the name, since
      * the salvager has a stupid partition ID parser */
@@ -1314,13 +1316,15 @@ DoSalvage(struct rx_connection * aconn, char * aparm1, char * aparm2,
 	goto done;
     }
     /* now wait for bnode to disappear */
+    count = 0;
     while (1) {
-	IOMGR_Sleep(5);
+	IOMGR_Sleep(1);
 	tp = tbuffer;
 	code = BOZO_GetInstanceInfo(aconn, "salvage-tmp", &tp, &istatus);
 	if (code)
 	    break;
-	printf("bos: waiting for salvage to complete.\n");
+	if ((++count % 5) == 0)
+	    printf("bos: waiting for salvage to complete.\n");
     }
     if (code != BZNOENT) {
 	printf("bos: salvage failed (%s)\n", em(code));
@@ -1674,8 +1678,12 @@ SalvageCmd(struct cmd_syndesc *as, void *arock)
 	confdir =
 	    (localauth ? AFSDIR_SERVER_ETC_DIRPATH :
 	     AFSDIR_CLIENT_ETC_DIRPATH);
-	code = vsu_ClientInit( /* noauth */ 1, confdir, tmpname,
-			      /* server auth */ 0, &cstruct, (int (*)())0);
+
+	code = vsu_ClientInit(confdir, tmpname,
+			      AFSCONF_SECOPTS_FALLBACK_NULL |
+			      AFSCONF_SECOPTS_NOAUTH,
+			      NULL, &cstruct);
+
 	if (code == 0) {
 	    newID = vsu_GetVolumeID(as->parms[2].items->data, cstruct, &err);
 	    if (newID == 0) {
