@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -9,22 +9,15 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
 #include <afs/stds.h>
-#if defined(AFS_LINUX24_ENV)
-#define _REGEX_RE_COMP
+
+#include <roken.h>
+
+#ifdef HAVE_POSIX_REGEX		/* use POSIX regexp library */
+#include <regex.h>
 #endif
-#include <sys/types.h>
+
 #include <afs/cmd.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#endif
-#include <errno.h>
 #include <afs/com_err.h>
 #include <afs/afsutil.h>
 #include <afs/budb.h>
@@ -36,15 +29,11 @@
 #include <afs/vldbint.h>	/* PA */
 #include <afs/ktime.h>		/* PA */
 #include <ubik.h>
-#include <time.h>
 #include <lock.h>
-#include <afs/butc.h>
 #include <afs/tcdata.h>
 #include <afs/butx.h>
 #include <afs/vsutils_prototypes.h>
-#ifdef HAVE_POSIX_REGEX		/* use POSIX regexp library */
-#include <regex.h>
-#endif
+
 #include "bc.h"
 #include "error_macros.h"
 #include "bucoord_internal.h"
@@ -70,9 +59,9 @@ static int EvalVolumeSet2(struct bc_config *aconfig, struct bc_volumeSet *avs,
 static int DBLookupByVolume(char *volumeName);
 
 int
-bc_EvalVolumeSet(struct bc_config *aconfig, 
-		 struct bc_volumeSet *avs, 
-		 struct bc_volumeDump **avols, 
+bc_EvalVolumeSet(struct bc_config *aconfig,
+		 struct bc_volumeSet *avs,
+		 struct bc_volumeDump **avols,
 		 struct ubik_client *uclient)
 {				/*bc_EvalVolumeSet */
     int code = -1;
@@ -104,9 +93,9 @@ struct serversort {
 };
 
 afs_int32
-getSPEntries(afs_uint32 server, afs_int32 partition, 
-	     struct serversort **serverlist, 
-	     struct serversort **ss, 
+getSPEntries(afs_uint32 server, afs_int32 partition,
+	     struct serversort **serverlist,
+	     struct serversort **ss,
 	     struct partitionsort **ps)
 {
     if (!(*ss) || ((*ss)->ipaddr != server)) {
@@ -118,13 +107,12 @@ getSPEntries(afs_uint32 server, afs_int32 partition,
     }
     /* No server entry added. Add one */
     if (!(*ss)) {
-	*ss = (struct serversort *)malloc(sizeof(struct serversort));
+	*ss = calloc(1, sizeof(struct serversort));
 	if (!(*ss)) {
 	    afs_com_err(whoami, BC_NOMEM, NULL);
 	    *ss = 0;
 	    return (BC_NOMEM);
 	}
-	memset(*ss, 0, sizeof(struct serversort));
 	(*ss)->ipaddr = server;
 	(*ss)->next = *serverlist;
 	*serverlist = *ss;
@@ -139,7 +127,7 @@ getSPEntries(afs_uint32 server, afs_int32 partition,
     }
     /* No partition entry added. Add one */
     if (!(*ps)) {
-	*ps = (struct partitionsort *)malloc(sizeof(struct partitionsort));
+	*ps = calloc(1, sizeof(struct partitionsort));
 	if (!(*ps)) {
 	    afs_com_err(whoami, BC_NOMEM, NULL);
 	    free(*ss);
@@ -147,7 +135,6 @@ getSPEntries(afs_uint32 server, afs_int32 partition,
 	    *ss = 0;
 	    return (BC_NOMEM);
 	}
-	memset(*ps, 0, sizeof(struct partitionsort));
 	(*ps)->part = partition;
 	(*ps)->next = (*ss)->partitions;
 	(*ss)->partitions = *ps;
@@ -156,7 +143,7 @@ getSPEntries(afs_uint32 server, afs_int32 partition,
 }
 
 afs_int32
-randSPEntries(struct serversort *serverlist, 
+randSPEntries(struct serversort *serverlist,
 	      struct bc_volumeDump **avols)
 {
     struct serversort *ss, **pss;
@@ -198,9 +185,9 @@ randSPEntries(struct serversort *serverlist,
 }
 
 static int
-EvalVolumeSet2(struct bc_config *aconfig, 
-	       struct bc_volumeSet *avs, 
-	       struct bc_volumeDump **avols, 
+EvalVolumeSet2(struct bc_config *aconfig,
+	       struct bc_volumeSet *avs,
+	       struct bc_volumeDump **avols,
 	       struct ubik_client *uclient)
 {				/*EvalVolumeSet2 */
     struct bc_volumeEntry *tve;
@@ -222,7 +209,7 @@ EvalVolumeSet2(struct bc_config *aconfig,
 
     /* For each of the volume set entries - collect the volumes that match it */
     for (tve = avs->ventries; tve; tve = tve->next) {
-	/* Put together a call to the vlserver for this vlentry. The 
+	/* Put together a call to the vlserver for this vlentry. The
 	 * performance gain is from letting the vlserver expand the
 	 * volumeset and not this routine.
 	 */
@@ -243,7 +230,7 @@ EvalVolumeSet2(struct bc_config *aconfig,
 	    bulkentries.nbulkentries_val = 0;
 	    nsi = -1;
 	    tcode =
-		ubik_Call(VL_ListAttributesN2, uclient, 0, &attributes,
+		ubik_VL_ListAttributesN2(uclient, 0, &attributes,
 			  tve->name, si, &nentries, &bulkentries, &nsi);
 	    if (tcode)
 		ERROR(tcode);
@@ -258,6 +245,11 @@ EvalVolumeSet2(struct bc_config *aconfig,
 	    if (nentries == 0)
 		ERROR(RXGEN_OPCODE);	/* Use EvalVolumeSet1 */
 
+	    if (nentries < 0)
+		nentries = 0;
+	    if (nentries > bulkentries.nbulkentries_len)
+		nentries = bulkentries.nbulkentries_len;
+
 	    /* Step through each entry and add it to the list of volumes */
 	    entries = bulkentries.nbulkentries_val;
 	    for (e = 0; e < nentries; e++) {
@@ -315,15 +307,13 @@ EvalVolumeSet2(struct bc_config *aconfig,
 
 		if (add) {
 		    /* Allocate a volume dump structure and its name */
-		    tvd = (struct bc_volumeDump *)
-			malloc(sizeof(struct bc_volumeDump));
+		    tvd = calloc(1, sizeof(struct bc_volumeDump));
 		    if (!tvd) {
 			afs_com_err(whoami, BC_NOMEM, NULL);
 			ERROR(BC_NOMEM);
 		    }
-		    memset(tvd, 0, sizeof(*tvd));
 
-		    tvd->name = (char *)malloc(strlen(entries[e].name) + 10);
+		    tvd->name = malloc(strlen(entries[e].name) + 10);
 		    if (!(tvd->name)) {
 			afs_com_err(whoami, BC_NOMEM, NULL);
 			free(tvd);
@@ -359,7 +349,7 @@ EvalVolumeSet2(struct bc_config *aconfig,
 
 	    /* Free memory allocated during VL call */
 	    if (bulkentries.nbulkentries_val) {
-		free((char *)bulkentries.nbulkentries_val);
+		free(bulkentries.nbulkentries_val);
 		bulkentries.nbulkentries_val = 0;
 		entries = 0;
 	    }
@@ -372,7 +362,7 @@ EvalVolumeSet2(struct bc_config *aconfig,
 
   error_exit:
     if (bulkentries.nbulkentries_val) {
-	free((char *)bulkentries.nbulkentries_val);
+	free(bulkentries.nbulkentries_val);
     }
     return (code);
 }				/*EvalVolumeSet2 */
@@ -381,29 +371,29 @@ EvalVolumeSet2(struct bc_config *aconfig,
  * EvalVolumeSetOld
  *
  * Description:
- *	Takes the entries in a volumeset and expands them into a list of 
+ *	Takes the entries in a volumeset and expands them into a list of
  *      volumes. Every VLDB volume entry is looked at and compared to the
  *      volumeset entries.
  *
- *      When matching a VLDB volume entry to a volumeset entry, 
+ *      When matching a VLDB volume entry to a volumeset entry,
  *       1. If the RW volume entry matches, that RW volume is used.
  *       2. Otherwise, if the BK volume entry matches, the BK volume is used.
  *       3. Finally, if the RO volume entry matches, the RO volume is used.
- *      For instance: A volumeset entry of ".* .* user.t.*" will match volume 
- *                    "user.troy" and "user.troy.backup". The rules will use 
+ *      For instance: A volumeset entry of ".* .* user.t.*" will match volume
+ *                    "user.troy" and "user.troy.backup". The rules will use
  *                    the RW volume "user.troy".
  *
  *      When a VLDB volume entry matches a volumeset entry (be it RW, BK or RO),
- *      that volume is used and matches against any remaining volumeset entries 
+ *      that volume is used and matches against any remaining volumeset entries
  *      are not even done.
  *      For instance: A 1st volumeset entry ".* .* .*.backup" will match with
- *                    "user.troy.backup". Its 2nd volumeset entry ".* .* .*" 
- *                    would have matched its RW volume "user.troy", but the first 
+ *                    "user.troy.backup". Its 2nd volumeset entry ".* .* .*"
+ *                    would have matched its RW volume "user.troy", but the first
  *                    match is used and the second match isn't even done.
  *
  * Arguments:
  *	aconfig	: Global configuration info.
- *	avs	: 
+ *	avs	:
  *	avols	: Ptr to linked list of entries describing volumes to dump.
  *	uclient	: Ptr to Ubik client structure.
  *
@@ -419,9 +409,9 @@ EvalVolumeSet2(struct bc_config *aconfig,
  *-----------------------------------------------------------------------------
  */
 static int
-EvalVolumeSet1(struct bc_config *aconfig, 
+EvalVolumeSet1(struct bc_config *aconfig,
 	       struct bc_volumeSet *avs,
-	       struct bc_volumeDump **avols, 
+	       struct bc_volumeDump **avols,
 	       struct ubik_client *uclient)
 {				/*EvalVolumeSet1 */
     afs_int32 code;		/*Result of various calls */
@@ -455,8 +445,7 @@ EvalVolumeSet1(struct bc_config *aconfig,
      */
     for (index = 0; 1; index = next_index) {	/*w */
 	memset(&entry, 0, sizeof(entry));
-	code = ubik_Call(VL_ListEntry,	/*Routine to invoke */
-			 uclient,	/*Ubik client structure */
+	code = ubik_VL_ListEntry(uclient,	/*Ubik client structure */
 			 0,	/*Ubik flags */
 			 index,	/*Current index */
 			 &count,	/*Ptr to working variable */
@@ -529,7 +518,7 @@ EvalVolumeSet1(struct bc_config *aconfig,
 			}
 		    }
 
-		    /* If the BK name matches the volume set entry, take 
+		    /* If the BK name matches the volume set entry, take
 		     * it and exit. Second choice is to use the BK volume.
 		     */
 		    if (entry.flags & BACK_EXISTS) {
@@ -586,15 +575,13 @@ EvalVolumeSet1(struct bc_config *aconfig,
 		}
 
 		total++;
-		tvd = (struct bc_volumeDump *)
-		    malloc(sizeof(struct bc_volumeDump));
+		tvd = calloc(1, sizeof(struct bc_volumeDump));
 		if (!tvd) {
 		    afs_com_err(whoami, BC_NOMEM, NULL);
 		    return (BC_NOMEM);
 		}
-		memset(tvd, 0, sizeof(*tvd));
 
-		tvd->name = (char *)malloc(strlen(entry.name) + 10);
+		tvd->name = malloc(strlen(entry.name) + 10);
 		if (!(tvd->name)) {
 		    afs_com_err(whoami, BC_NOMEM, NULL);
 		    free(tvd);
@@ -736,23 +723,20 @@ bc_FloatATOI(char *anum)
 char *
 bc_CopyString(char *astring)
 {
-    afs_int32 tlen;
     char *tp;
 
     if (!astring)
 	return (NULL);		/* propagate null strings easily */
-    tlen = strlen(astring);
-    tp = (char *)malloc(tlen + 1);	/* don't forget the terminating null */
+    tp = strdup(astring);
     if (!tp) {
 	afs_com_err(whoami, BC_NOMEM, NULL);
 	return (tp);
     }
-    strcpy(tp, astring);
     return tp;
 }
 
 /* concatParams
- * 
+ *
  *    Concatenates the parameters of an option and returns the string.
  *
  */
@@ -775,7 +759,7 @@ concatParams(struct cmd_item *itemPtr)
 	return (NULL);
     }
 
-    string = (char *)malloc(length);	/* allocate the string */
+    string = malloc(length);	/* allocate the string */
     if (!string) {
 	afs_com_err(whoami, BC_NOMEM, NULL);
 	return (NULL);
@@ -796,7 +780,7 @@ concatParams(struct cmd_item *itemPtr)
 /* printIfStatus
  *	print out an interface status node as received from butc
  */
- 
+
 void
 printIfStatus(struct tciStatusS *statusPtr)
 {
@@ -921,11 +905,10 @@ bc_WaitForNoJobs(void)
 {
     int i;
     int usefulJobRunning = 1;
+    int printWaiting = 1;
 
     extern dlqlinkT statusHead;
 
-    afs_com_err(whoami, 0, "waiting for job termination");
-
     while (usefulJobRunning) {
 	usefulJobRunning = (dlqEmpty(&statusHead) ? 0 : 1);
 	if (dispatchLock.excl_locked)
@@ -936,8 +919,13 @@ bc_WaitForNoJobs(void)
 	}
 
 	/* Wait 5 seconds and check again */
-	if (usefulJobRunning)
+	if (usefulJobRunning) {
+            if (printWaiting) {
+                afs_com_err(whoami, 0, "waiting for job termination");
+                printWaiting = 0;
+            }
 	    IOMGR_Sleep(5);
+        }
     }
     return (lastTaskCode);
 }
@@ -1003,7 +991,7 @@ bc_JobsCmd(struct cmd_syndesc *as, void *arock)
 	}
     }
 
-    /* 
+    /*
      * Now print the scheduled dumps.
      */
     if (!dlqEmpty(&statusHead) && as)
@@ -1127,7 +1115,6 @@ bc_KillCmd(struct cmd_syndesc *as, void *arock)
 	}
 	statusPtr->flags |= ABORT_REQUEST;
 	unlock_Status();
-	return (0);
     }
     return 0;
 }
@@ -1193,14 +1180,13 @@ bc_VolRestoreCmd(struct cmd_syndesc *as, void *arock)
 
     for (ti = as->parms[2].items; ti; ti = ti->next) {
 	/* build list of volume items */
-	tvol = (struct bc_volumeDump *)malloc(sizeof(struct bc_volumeDump));
+	tvol = calloc(1, sizeof(struct bc_volumeDump));
 	if (!tvol) {
 	    afs_com_err(whoami, BC_NOMEM, NULL);
 	    return BC_NOMEM;
 	}
-	memset(tvol, 0, sizeof(struct bc_volumeDump));
 
-	tvol->name = (char *)malloc(VOLSER_MAXVOLNAME + 1);
+	tvol->name = malloc(VOLSER_MAXVOLNAME + 1);
 	if (!tvol->name) {
 	    afs_com_err(whoami, BC_NOMEM, NULL);
 	    return BC_NOMEM;
@@ -1235,12 +1221,12 @@ bc_VolRestoreCmd(struct cmd_syndesc *as, void *arock)
     oldFlag = 0;
 
     /* Read all the port offsets into the ports array. The first element in the
-     * array is for full restore and the rest are for incremental restores 
+     * array is for full restore and the rest are for incremental restores
      */
     if (as->parms[5].items) {
 	for (ti = as->parms[5].items; ti; ti = ti->next)
 	    portCount++;
-	ports = (afs_int32 *) malloc(portCount * sizeof(afs_int32));
+	ports = malloc(portCount * sizeof(afs_int32));
 	if (!ports) {
 	    afs_com_err(whoami, BC_NOMEM, NULL);
 	    return BC_NOMEM;
@@ -1261,7 +1247,7 @@ bc_VolRestoreCmd(struct cmd_syndesc *as, void *arock)
 	if (dumpID <= 0)
 	  dumpID = 0;
       }
-    
+
     /*
      * Perform the call to start the restore.
      */
@@ -1309,7 +1295,7 @@ bc_DiskRestoreCmd(struct cmd_syndesc *as, void *arock)
 
     /* parm 0 is the server to restore
      * parm 1 is the partition to restore
-     
+
      * parm 8 and above as in VolRestoreCmd:
      * parm 8 is the new server to restore to
      * parm 9 is the new partition to restore to
@@ -1370,7 +1356,7 @@ bc_DiskRestoreCmd(struct cmd_syndesc *as, void *arock)
     if (as->parms[2].items) {
 	for (ti = as->parms[2].items; ti; ti = ti->next)
 	    portCount++;
-	ports = (afs_int32 *) malloc(portCount * sizeof(afs_int32));
+	ports = malloc(portCount * sizeof(afs_int32));
 	if (!ports) {
 	    afs_com_err(whoami, BC_NOMEM, NULL);
 	    return BC_NOMEM;
@@ -1397,7 +1383,7 @@ bc_DiskRestoreCmd(struct cmd_syndesc *as, void *arock)
 	return (-1);
     }
 
-    /* Since we want only RW volumes, remove any 
+    /* Since we want only RW volumes, remove any
      * BK or RO volumes from the list.
      */
     for (prev = 0, tvol = volsToRestore; tvol; tvol = nextvol) {
@@ -1537,11 +1523,9 @@ bc_VolsetRestoreCmd(struct cmd_syndesc *as, void *arock)
 	    }
 
 	    /* Allocate a volumeDump structure and link it in */
-	    tvol =
-		(struct bc_volumeDump *)malloc(sizeof(struct bc_volumeDump));
-	    memset(tvol, 0, sizeof(struct bc_volumeDump));
+	    tvol = calloc(1, sizeof(struct bc_volumeDump));
 
-	    tvol->name = (char *)malloc(VOLSER_MAXVOLNAME + 1);
+	    tvol->name = malloc(VOLSER_MAXVOLNAME + 1);
 	    if (!tvol->name) {
 		afs_com_err(whoami, BC_NOMEM, NULL);
 		return BC_NOMEM;
@@ -1567,7 +1551,7 @@ bc_VolsetRestoreCmd(struct cmd_syndesc *as, void *arock)
     if (as->parms[2].items) {
 	for (ti = as->parms[2].items; ti; ti = ti->next)
 	    portCount++;
-	ports = (afs_int32 *) malloc(portCount * sizeof(afs_int32));
+	ports = malloc(portCount * sizeof(afs_int32));
 	if (!ports) {
 	    afs_com_err(whoami, BC_NOMEM, NULL);
 	    return BC_NOMEM;
@@ -1640,7 +1624,6 @@ bc_DumpCmd(struct cmd_syndesc *as, void *arock)
     afs_int32 problemFindingDump;	/* can't find parent(s) */
 
     afs_int32 *portp = NULL;
-    afs_int32 portCount = 0;
     afs_int32 doAt, atTime;	/* Time a timed-dump is to start at */
     afs_int32 length;
     char *timeString;
@@ -1668,10 +1651,10 @@ bc_DumpCmd(struct cmd_syndesc *as, void *arock)
 	return (code);
     }
 
-    /* 
+    /*
      * Some parameters cannot be specified together
-     * The "-file" option cannot exist with the "-volume", "-dump", 
-     * "-portoffset", or "-append" option 
+     * The "-file" option cannot exist with the "-volume", "-dump",
+     * "-portoffset", or "-append" option
      */
     if (as->parms[6].items) {
 	loadfile = 1;
@@ -1689,7 +1672,7 @@ bc_DumpCmd(struct cmd_syndesc *as, void *arock)
 	}
     }
 
-    /* 
+    /*
      * Get the time we are to perform this dump
      */
     if (as->parms[3].items) {
@@ -1714,7 +1697,7 @@ bc_DumpCmd(struct cmd_syndesc *as, void *arock)
 
     dontExecute = (as->parms[5].items ? 1 : 0);	/* -n */
 
-    /* 
+    /*
      * If this dump is not a load file, then check the parameters.
      */
     if (!loadfile) {		/*6 */
@@ -1723,8 +1706,7 @@ bc_DumpCmd(struct cmd_syndesc *as, void *arock)
 
 	/* get the port number, if one was specified */
 	if (as->parms[2].items) {
-	    portCount = 1;
-	    portp = (afs_int32 *) malloc(sizeof(afs_int32));
+	    portp = malloc(sizeof(afs_int32));
 	    if (!portp) {
 		afs_com_err(whoami, BC_NOMEM, NULL);
 		return BC_NOMEM;
@@ -1758,12 +1740,12 @@ bc_DumpCmd(struct cmd_syndesc *as, void *arock)
 
     /*6 */
     /*
-     * If given the "-at" option, then add this to the jobs list and return 
+     * If given the "-at" option, then add this to the jobs list and return
      * with no error.
      *
      * Create a status node for this timed dump.
      * Fill in the time to dump and the cmd line for the dump leaving off
-     * the -at option.  If the -n option is there, it is scheduled with 
+     * the -at option.  If the -n option is there, it is scheduled with
      * the Timed dump as opposed to not scheduling the time dump at all.
      */
     if (doAt) {
@@ -1805,7 +1787,7 @@ bc_DumpCmd(struct cmd_syndesc *as, void *arock)
 	    sprintf(statusPtr->taskName, "Scheduled Dump");
 	    statusPtr->jobNumber = bc_jobNumber();
 	    statusPtr->scheduledDump = atTime;
-	    statusPtr->cmdLine = (char *)malloc(length);
+	    statusPtr->cmdLine = malloc(length);
 	    if (!statusPtr->cmdLine) {
 		afs_com_err(whoami, BC_NOMEM, NULL);
 		return BC_NOMEM;
@@ -1851,22 +1833,21 @@ bc_DumpCmd(struct cmd_syndesc *as, void *arock)
 	return (0);
     }
 
-    /* 
+    /*
      * Read and execute the load file if specified.  The work of reading is done
      * in the main routine prior the dispatch call. loadFile and dontExecute are
      * global variables so this can take place in main.
      */
     if (loadfile) {
-	loadFile = (char *)malloc(strlen(as->parms[6].items->data) + 1);
+	loadFile = strdup(as->parms[6].items->data);
 	if (!loadFile) {
 	    afs_com_err(whoami, BC_NOMEM, NULL);
 	    return BC_NOMEM;
 	}
-	strcpy(loadFile, as->parms[6].items->data);
 	return 0;
     }
 
-    /* 
+    /*
      * We are doing a real dump (no load file or timed dump).
      */
     printf("Starting dump of volume set '%s' (dump level '%s')\n", vsName,
@@ -1889,9 +1870,9 @@ bc_DumpCmd(struct cmd_syndesc *as, void *arock)
 	}
 
 	/* We found the most recent dump at this level. Now check
-	 * if we should use it by seeing if its full dump hierarchy 
+	 * if we should use it by seeing if its full dump hierarchy
 	 * exists. If it doesn't, we don't want to base our incremental
-	 * off of this dump. 
+	 * off of this dump.
 	 */
 	if (!parent || (dumpEntry.id > parent)) {
 	    /* Follow the parent dumps to see if they are all there */
@@ -1917,7 +1898,7 @@ bc_DumpCmd(struct cmd_syndesc *as, void *arock)
 	}
     }
 
-    /* If the problemflag was raise, it means we are not doing the 
+    /* If the problemflag was raise, it means we are not doing the
      * dump at the level we requested it be done at.
      */
     if (problemFindingDump) {
@@ -2179,7 +2160,7 @@ bc_ScanDumpsCmd(struct cmd_syndesc *as, void *arock)
  */
 
 afs_int32
-bc_ParseExpiration(struct cmd_parmdesc *paramPtr, afs_int32 *expType, 
+bc_ParseExpiration(struct cmd_parmdesc *paramPtr, afs_int32 *expType,
 		   afs_int32 *expDate)
 {
     struct cmd_item *itemPtr;
@@ -2280,7 +2261,7 @@ bc_dbVerifyCmd(struct cmd_syndesc *as, void *arock)
     detail = (as->parms[0].items ? 1 : 0);	/* print more details */
 
     code =
-	ubik_Call(BUDB_DbVerify, udbHandle.uh_client, 0, &status, &orphans,
+	ubik_BUDB_DbVerify(udbHandle.uh_client, 0, &status, &orphans,
 		  &host);
 
     if (code) {
@@ -2348,7 +2329,7 @@ deleteDump(afs_uint32 dumpid,		/* The dumpid to delete */
 
 	/* If dump is to an XBSA server, connect to butc and send it
 	 * the dump to delete. Butc will contact the XBSA server.
-	 * The dump will not be an appended dump because XBSA butc 
+	 * The dump will not be an appended dump because XBSA butc
 	 * does not support the append option.
 	 */
 	if (xbsadump && dumpEntry.nVolumes) {
@@ -2477,7 +2458,12 @@ bc_deleteDumpCmd(struct cmd_syndesc *as, void *arock)
     if (havegroupid)
 	groupId = atoi(as->parms[4].items->data);
 
-    noexecute = (as->parms[7].items ? 1 : 0);
+    if (as->parms[7].items || as->parms[8].items) {
+        /* -noexecute (hidden) or -dryrun used */
+	noexecute = 1;
+    } else {
+	noexecute = 0;
+    }
 
     /* Get the time to delete to */
     if (as->parms[2].items) {	/* -to */
@@ -2691,7 +2677,7 @@ bc_restoreDbCmd(struct cmd_syndesc *as, void *arock)
 }
 
 /* ----------------------------------
- * supporting routines for database examination 
+ * supporting routines for database examination
  * ----------------------------------
  */
 
@@ -2766,12 +2752,10 @@ DBLookupByVolume(char *volumeName)
 	    for (i = 0; i < numEntries; i++) {	/*f */
 		struct dumpedVol *insPtr, **prevPtr;
 
-		tempPtr =
-		    (struct dumpedVol *)malloc(sizeof(struct dumpedVol));
+		tempPtr = calloc(1, sizeof(struct dumpedVol));
 		if (!tempPtr)
 		    ERROR(BC_NOMEM);
 
-		memset(tempPtr, 0, sizeof(*tempPtr));
 		tempPtr->incTime = volumeEntry[i].clone;
 		tempPtr->dumpID = volumeEntry[i].dump;
 		strncpy(tempPtr->tapeName, volumeEntry[i].tape,
@@ -2924,13 +2908,12 @@ dumpInfo(afs_int32 dumpid, afs_int32 detailFlag)
 
     /* now get the list of tapes */
     for (tapeNumber = dumpEntry.tapes.b; tapeNumber <= dumpEntry.tapes.maxTapes; tapeNumber++) {	/*f */
-	tapeLinkPtr = (struct tapeLink *)malloc(sizeof(struct tapeLink));
+	tapeLinkPtr = calloc(1, sizeof(struct tapeLink));
 	if (!tapeLinkPtr) {
 	    afs_com_err(whoami, BC_NOMEM, NULL);
 	    ERROR(BC_NOMEM);
 	}
 
-	memset(tapeLinkPtr, 0, sizeof(*tapeLinkPtr));
 	code = bcdb_FindTapeSeq(dumpid, tapeNumber, &tapeLinkPtr->tapeEntry);
 	if (code) {
 	    code = 0;
@@ -2975,13 +2958,11 @@ dumpInfo(afs_int32 dumpid, afs_int32 detailFlag)
 	    for (i = 0; i < vl.budb_volumeList_len; i++) {
 		link = &tapeLinkPtr->firstVolume;
 
-		volumeLinkPtr =
-		    (struct volumeLink *)malloc(sizeof(struct volumeLink));
+		volumeLinkPtr = calloc(1, sizeof(struct volumeLink));
 		if (!volumeLinkPtr) {
 		    afs_com_err(whoami, BC_NOMEM, NULL);
 		    ERROR(BC_NOMEM);
 		}
-		memset(volumeLinkPtr, 0, sizeof(*volumeLinkPtr));
 
 		memcpy(&volumeLinkPtr->volumeEntry,
 		       &vl.budb_volumeList_val[i],
@@ -3095,7 +3076,7 @@ printRecentDumps(int ndumps)
 	dl.budb_dumpList_val = 0;
 
 	/* outline algorithm */
-	code = ubik_Call(BUDB_GetDumps, udbHandle.uh_client, 0, BUDB_MAJORVERSION, BUDB_OP_NPREVIOUS, "",	/* no name */
+	code = ubik_BUDB_GetDumps(udbHandle.uh_client, 0, BUDB_MAJORVERSION, BUDB_OP_NPREVIOUS, "",	/* no name */
 			 0,	/* start */
 			 ndumps,	/* end */
 			 index,	/* index */
@@ -3137,7 +3118,7 @@ printRecentDumps(int ndumps)
     return (code);
 }
 
-/* bc_dumpInfoCmd 
+/* bc_dumpInfoCmd
  *	list the dumps and contens of the dumps.
  * params:
  *	as - name of tape
