@@ -13,6 +13,7 @@
 
 #include <roken.h>
 
+#include <afs/opr.h>
 #include <afs/com_err.h>
 #include <afs/bubasics.h>
 
@@ -86,11 +87,10 @@ createStatusNode(void)
 {
     statusP ptr;
 
-    ptr = (statusP) malloc(sizeof(*ptr));
+    ptr = calloc(1, sizeof(*ptr));
     if (ptr == 0) {
 	return (0);
     }
-    memset(ptr, 0, sizeof(*ptr));
 
     /* link it onto the chain of status entries */
     ObtainWriteLock(&statusQueueLock);
