@@ -529,17 +529,13 @@ WalkNextChain(char map[],		/* one byte per db entry */
 	    if (id == PRBADID)
 		continue;
 	    else if (id) {
-		int eid_s, id_s;
+		int id_s;
 		sgcount++;
 		/* in case the ids are large, convert to pure sign. */
 		if (id > 0)
 		    id_s = 1;
 		else
 		    id_s = -1;
-		if (eid > 0)
-		    eid_s = 1;
-		else
-		    eid_s = -1;
 		if (id_s > 0) {
 		    fprintf(stderr,
 			    "User can't be member of supergroup list\n");
@@ -962,10 +958,7 @@ QuoteName(char *s)
 {
     char *qs;
     if (strpbrk(s, " \t")) {
-	qs = (char *)malloc(strlen(s) + 3);
-	strcpy(qs, "\"");
-	strcat(qs, s);
-	strcat(qs, "\"");
+	asprintf(&qs, "\"%s\"", s);
     } else
 	qs = s;
     return qs;
@@ -1256,8 +1249,7 @@ CheckPrDatabase(struct misc_data *misc)	/* info & statistics */
     }
     if (misc->verbose)
 	printf("Database has %d entries\n", n);
-    map = (char *)malloc(n);
-    memset(map, 0, n);
+    map = calloc(1, n);
     misc->nEntries = n;
 
     if (misc->verbose) {
@@ -1285,14 +1277,13 @@ CheckPrDatabase(struct misc_data *misc)	/* info & statistics */
 #else
     n = ((misc->maxId > misc->maxForId) ? misc->maxId : misc->maxForId);
     misc->idRange = n - misc->minId + 1;
-    misc->idmap = (afs_int32 *) malloc(misc->idRange * sizeof(afs_int32));
+    misc->idmap = calloc(misc->idRange, sizeof(afs_int32));
     if (!misc->idmap) {
 	afs_com_err(whoami, 0, "Unable to malloc space for max ids of %d",
 		misc->idRange);
 	code = -1;
 	goto abort;
     }
-    memset(misc->idmap, 0, misc->idRange * sizeof(misc->idmap[0]));
 #endif /* SUPERGROUPS */
 
     if (misc->verbose) {
@@ -1483,12 +1474,11 @@ inccount(struct idused **idmapp, int id)
 	idmapp = &idmap->idnext;
     }
     if (!idmap) {
-	idmap = (struct idused *)malloc(sizeof *idmap);
+	idmap = calloc(1, sizeof *idmap);
 	if (!idmap) {
 	    perror("idmap");
 	    exit(1);
 	}
-	memset(idmap, 0, sizeof idmap);
 	idmap->idstart = id & ~(IDCOUNT - 1);
 	idmap->idnext = *idmapp;
 	*idmapp = idmap;
