@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,58 +10,63 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
-#include <sys/types.h>
-#include <string.h>
-#include <stdarg.h>
-#include <errno.h>
-
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#include <time.h>
+#include <afs/opr.h>
+#ifdef AFS_PTHREAD_ENV
+# include <opr/lock.h>
 #else
-#include <sys/file.h>
-#include <sys/time.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
+# include <opr/lockstub.h>
 #endif
 
 #include <lock.h>
-#include <rx/xdr.h>
 #include <rx/rx.h>
+#include <rx/rxkad.h>
 #include <rx/rx_multi.h>
 #include <afs/cellconfig.h>
 #ifndef AFS_NT40_ENV
 #include <afs/afsutil.h>
-#include <afs/netutils.h>
 #endif
 
 #define UBIK_INTERNALS
 #include "ubik.h"
 #include "ubik_int.h"
 
+/* These global variables were used to set the function to use to initialise
+ * the client security layer. They are retained for backwards compatiblity with
+ * legacy callers - the ubik_SetClientSecurityProcs() interface should be used
+ * instead
+ */
+int (*ubik_CRXSecurityProc) (void *rock, struct rx_securityClass **,
+                             afs_int32 *);
+void *ubik_CRXSecurityRock;
+
 /*! \name statics used to determine if we're the sync site */
-static afs_int32 syncSiteUntil = 0;	/*!< valid only if amSyncSite */
-int ubik_amSyncSite = 0;	/*!< flag telling if I'm sync site */
 static int nServers;		/*!< total number of servers */
 static char amIMagic = 0;	/*!< is this host the magic host */
 char amIClone = 0;		/*!< is this a clone which doesn't vote */
 static char ubik_singleServer = 0;
 /*\}*/
-int (*ubik_CRXSecurityProc) (void *rock, struct rx_securityClass **,
-			     afs_int32 *);
-void *ubik_CRXSecurityRock;
+static int (*secLayerProc) (void *rock, struct rx_securityClass **,
+			    afs_int32 *) = NULL;
+static int (*tokenCheckProc) (void *rock) = NULL;
+static void * securityRock = NULL;
+
 afs_int32 ubikSecIndex;
 struct rx_securityClass *ubikSecClass;
-static int ubeacon_InitServerListCommon(afs_int32 ame,
+
+/* Values protected by the address lock */
+struct addr_data addr_globals;
+
+/* Values protected by the beacon lock */
+struct beacon_data beacon_globals;
+
+static int ubeacon_InitServerListCommon(afs_uint32 ame,
 					struct afsconf_cell *info,
 					char clones[],
-					afs_int32 aservers[]);
+					afs_uint32 aservers[]);
 static int verifyInterfaceAddress(afs_uint32 *ame, struct afsconf_cell *info,
 				  afs_uint32 aservers[]);
-static int updateUbikNetworkAddress(afs_uint32 ubik_host[UBIK_MAX_INTERFACE_ADDR]);
-
 
 /*! \file
  * Module responsible for both deciding if we're currently the sync site,
@@ -80,7 +85,7 @@ static int updateUbikNetworkAddress(afs_uint32 ubik_host[UBIK_MAX_INTERFACE_ADDR
  * not to vote for someone else.  This parameter (BIG seconds) is not actually passed in
  * the interface (perhaps it should be?) but is instead a compile time constant that both
  * sides know about.
- 
+
  * The beacon and vote modules work intimately together; the vote module decides how long
  * it should promise the beacon module its vote, and the beacon module takes all of these
  * votes and decides for how long it is the synchronization site.
@@ -88,13 +93,41 @@ static int updateUbikNetworkAddress(afs_uint32 ubik_host[UBIK_MAX_INTERFACE_ADDR
 
 /*! \brief procedure called from debug rpc call to get this module's state for debugging */
 void
-ubeacon_Debug(register struct ubik_debug *aparm)
+ubeacon_Debug(struct ubik_debug *aparm)
 {
     /* fill in beacon's state fields in the ubik_debug structure */
-    aparm->syncSiteUntil = syncSiteUntil;
+    aparm->syncSiteUntil = beacon_globals.syncSiteUntil;
     aparm->nServers = nServers;
 }
 
+static int
+amSyncSite(void) {
+    afs_int32 now;
+    afs_int32 rcode;
+
+    /* special case for fast startup */
+    if (nServers == 1 && !amIClone)
+	return 1;		/* one guy is always the sync site */
+
+    UBIK_BEACON_LOCK;
+    if (beacon_globals.ubik_amSyncSite == 0 || amIClone)
+	rcode = 0;		/* if I don't think I'm the sync site, say so */
+    else {
+	now = FT_ApproxTime();
+	if (beacon_globals.syncSiteUntil <= now) {	/* if my votes have expired, say so */
+	    if (beacon_globals.ubik_amSyncSite)
+		ubik_dprint("Ubik: I am no longer the sync site\n");
+	    beacon_globals.ubik_amSyncSite = 0;
+	    rcode = 0;
+	} else {
+	    rcode = 1;		/* otherwise still have the required votes */
+	}
+    }
+    UBIK_BEACON_UNLOCK;
+    ubik_dprint("beacon: amSyncSite is %d\n", rcode);
+    return rcode;
+}
+
 /*!
  * \brief Procedure that determines whether this site has enough current votes to remain sync site.
  *
@@ -114,30 +147,13 @@ ubeacon_Debug(register struct ubik_debug *aparm)
 int
 ubeacon_AmSyncSite(void)
 {
-    register afs_int32 now;
-    register afs_int32 rcode;
+    afs_int32 rcode;
 
-    /* special case for fast startup */
-    if (nServers == 1 && !amIClone) {
-	return 1;		/* one guy is always the sync site */
-    }
+    rcode = amSyncSite();
+
+    if (!rcode)
+	urecovery_ResetState();
 
-    if (ubik_amSyncSite == 0 || amIClone)
-	rcode = 0;		/* if I don't think I'm the sync site, say so */
-    else {
-	now = FT_ApproxTime();
-	if (syncSiteUntil <= now) {	/* if my votes have expired, say so */
-	    if (ubik_amSyncSite)
-		ubik_dprint("Ubik: I am no longer the sync site\n");
-	    ubik_amSyncSite = 0;
-	    rcode = 0;
-	} else {
-	    rcode = 1;		/* otherwise still have the required votes */
-	}
-    }
-    if (rcode == 0)
-	urecovery_ResetState();	/* force recovery to re-execute */
-    ubik_dprint("beacon: amSyncSite is %d\n", rcode);
     return rcode;
 }
 
@@ -145,7 +161,7 @@ ubeacon_AmSyncSite(void)
  * \see ubeacon_InitServerListCommon()
  */
 int
-ubeacon_InitServerListByInfo(afs_int32 ame, struct afsconf_cell *info, 
+ubeacon_InitServerListByInfo(afs_uint32 ame, struct afsconf_cell *info,
 			     char clones[])
 {
     afs_int32 code;
@@ -161,7 +177,7 @@ ubeacon_InitServerListByInfo(afs_int32 ame, struct afsconf_cell *info,
  * \see ubeacon_InitServerListCommon()
  */
 int
-ubeacon_InitServerList(afs_int32 ame, register afs_int32 aservers[])
+ubeacon_InitServerList(afs_uint32 ame, afs_uint32 aservers[])
 {
     afs_int32 code;
 
@@ -171,14 +187,65 @@ ubeacon_InitServerList(afs_int32 ame, register afs_int32 aservers[])
     return code;
 }
 
+/* Must be called with address lock held */
+void
+ubeacon_InitSecurityClass(void)
+{
+    int i;
+    /* get the security index to use, if we can */
+    if (secLayerProc) {
+	i = (*secLayerProc) (securityRock, &addr_globals.ubikSecClass, &addr_globals.ubikSecIndex);
+    } else if (ubik_CRXSecurityProc) {
+	i = (*ubik_CRXSecurityProc) (ubik_CRXSecurityRock, &addr_globals.ubikSecClass,
+				     &addr_globals.ubikSecIndex);
+    } else
+	i = 1;
+    if (i) {
+	/* don't have sec module yet */
+	addr_globals.ubikSecIndex = 0;
+	addr_globals.ubikSecClass = rxnull_NewClientSecurityObject();
+    }
+}
+
+void
+ubeacon_ReinitServer(struct ubik_server *ts)
+{
+    if (tokenCheckProc && !(*tokenCheckProc) (securityRock)) {
+	struct rx_connection *disk_rxcid;
+	struct rx_connection *vote_rxcid;
+	struct rx_connection *tmp;
+	UBIK_ADDR_LOCK;
+	ubeacon_InitSecurityClass();
+	disk_rxcid =
+	    rx_NewConnection(rx_HostOf(rx_PeerOf(ts->disk_rxcid)),
+			     ubik_callPortal, DISK_SERVICE_ID,
+			     addr_globals.ubikSecClass, addr_globals.ubikSecIndex);
+	if (disk_rxcid) {
+	    tmp = ts->disk_rxcid;
+	    ts->disk_rxcid = disk_rxcid;
+	    rx_PutConnection(tmp);
+	}
+	vote_rxcid =
+	    rx_NewConnection(rx_HostOf(rx_PeerOf(ts->vote_rxcid)),
+			     ubik_callPortal, VOTE_SERVICE_ID,
+			     addr_globals.ubikSecClass, addr_globals.ubikSecIndex);
+	if (vote_rxcid) {
+	    tmp = ts->vote_rxcid;
+	    ts->vote_rxcid = vote_rxcid;
+	    rx_PutConnection(tmp);
+	}
+	UBIK_ADDR_UNLOCK;
+    }
+}
+
 /*!
  * \brief setup server list
  *
  * \param ame "address of me"
  * \param aservers list of other servers
  *
- * called only at initialization to set up the list of servers to 
- * contact for votes.  Just creates the server structure.  
+ * called only at initialization to set up the list of servers to
+ * contact for votes.  Just creates the server structure.
  *
  * The "magic" host is the one with the lowest internet address.  It is
  * magic because its vote counts epsilon more than the others.  This acts
@@ -188,22 +255,22 @@ ubeacon_InitServerList(afs_int32 ame, register afs_int32 aservers[])
  * site system, we'd be out of business.
  *
  * \note There are two connections in every server structure, one for
- * vote calls (which must always go through quickly) and one for database 
- * operations, which are subject to waiting for locks.  If we used only 
- * one, the votes would sometimes get held up behind database operations, 
- * and the sync site guarantees would timeout even though the host would be 
+ * vote calls (which must always go through quickly) and one for database
+ * operations, which are subject to waiting for locks.  If we used only
+ * one, the votes would sometimes get held up behind database operations,
+ * and the sync site guarantees would timeout even though the host would be
  * up for communication.
  *
  * \see ubeacon_InitServerList(), ubeacon_InitServerListByInfo()
  */
 int
-ubeacon_InitServerListCommon(afs_int32 ame, struct afsconf_cell *info, 
-			     char clones[], register afs_int32 aservers[])
+ubeacon_InitServerListCommon(afs_uint32 ame, struct afsconf_cell *info,
+			     char clones[], afs_uint32 aservers[])
 {
-    register struct ubik_server *ts;
+    struct ubik_server *ts;
     afs_int32 me = -1;
-    register afs_int32 servAddr;
-    register afs_int32 i, code;
+    afs_int32 servAddr;
+    afs_int32 i, code;
     afs_int32 magicHost;
     struct ubik_server *magicServer;
 
@@ -211,17 +278,8 @@ ubeacon_InitServerListCommon(afs_int32 ame, struct afsconf_cell *info,
     if ((code = verifyInterfaceAddress(&ame, info, aservers)))
 	return code;
 
-    /* get the security index to use, if we can */
-    if (ubik_CRXSecurityProc) {
-	i = (*ubik_CRXSecurityProc) (ubik_CRXSecurityRock, &ubikSecClass,
-				     &ubikSecIndex);
-    } else
-	i = 1;
-    if (i) {
-	/* don't have sec module yet */
-	ubikSecIndex = 0;
-	ubikSecClass = rxnull_NewClientSecurityObject();
-    }
+    ubeacon_InitSecurityClass();
+
     magicHost = ntohl(ame);	/* do comparisons in host order */
     magicServer = (struct ubik_server *)0;
 
@@ -240,8 +298,7 @@ ubeacon_InitServerListCommon(afs_int32 ame, struct afsconf_cell *info,
 	for (i = 0; i < info->numServers; i++) {
 	    if (i == me)
 		continue;
-	    ts = (struct ubik_server *)malloc(sizeof(struct ubik_server));
-	    memset(ts, 0, sizeof(struct ubik_server));
+	    ts = calloc(1, sizeof(struct ubik_server));
 	    ts->next = ubik_servers;
 	    ubik_servers = ts;
 	    ts->addr[0] = info->hostAddr[i].sin_addr.s_addr;
@@ -260,12 +317,12 @@ ubeacon_InitServerListCommon(afs_int32 ame, struct afsconf_cell *info,
 	    ts->vote_rxcid =
 		rx_NewConnection(info->hostAddr[i].sin_addr.s_addr,
 				 ubik_callPortal, VOTE_SERVICE_ID,
-				 ubikSecClass, ubikSecIndex);
+				 addr_globals.ubikSecClass, addr_globals.ubikSecIndex);
 	    /* for disk reqs */
 	    ts->disk_rxcid =
 		rx_NewConnection(info->hostAddr[i].sin_addr.s_addr,
 				 ubik_callPortal, DISK_SERVICE_ID,
-				 ubikSecClass, ubikSecIndex);
+				 addr_globals.ubikSecClass, addr_globals.ubikSecIndex);
 	    ts->up = 1;
 	}
     } else {
@@ -273,13 +330,14 @@ ubeacon_InitServerListCommon(afs_int32 ame, struct afsconf_cell *info,
 	while ((servAddr = *aservers++)) {
 	    if (i >= MAXSERVERS)
 		return UNHOSTS;	/* too many hosts */
-	    ts = (struct ubik_server *)malloc(sizeof(struct ubik_server));
-	    memset(ts, 0, sizeof(struct ubik_server));
+	    ts = calloc(1, sizeof(struct ubik_server));
 	    ts->next = ubik_servers;
 	    ubik_servers = ts;
 	    ts->addr[0] = servAddr;	/* primary address in  net byte order */
-	    ts->vote_rxcid = rx_NewConnection(servAddr, ubik_callPortal, VOTE_SERVICE_ID, ubikSecClass, ubikSecIndex);	/* for vote reqs */
-	    ts->disk_rxcid = rx_NewConnection(servAddr, ubik_callPortal, DISK_SERVICE_ID, ubikSecClass, ubikSecIndex);	/* for disk reqs */
+	    ts->vote_rxcid = rx_NewConnection(servAddr, ubik_callPortal, VOTE_SERVICE_ID,
+			addr_globals.ubikSecClass, addr_globals.ubikSecIndex);	/* for vote reqs */
+	    ts->disk_rxcid = rx_NewConnection(servAddr, ubik_callPortal, DISK_SERVICE_ID,
+			addr_globals.ubikSecClass, addr_globals.ubikSecIndex);	/* for disk reqs */
 	    ts->isClone = 0;	/* don't know about clones */
 	    ts->up = 1;
 	    if (ntohl((afs_uint32) servAddr) < (afs_uint32) magicHost) {
@@ -301,21 +359,17 @@ ubeacon_InitServerListCommon(afs_int32 ame, struct afsconf_cell *info,
 	nServers = i + 1;	/* count this server as well as the remotes */
 
     ubik_quorum = (nServers >> 1) + 1;	/* compute the majority figure */
-    /* send addrs to all other servers */
-    code = updateUbikNetworkAddress(ubik_host);
-    if (code)
-	return code;
 
 /* Shoud we set some defaults for RX??
-    r_retryInterval = 2;	
+    r_retryInterval = 2;
     r_nRetries = (RPCTIMEOUT/r_retryInterval);
 */
     if (info) {
 	if (!ubik_servers)	/* special case 1 server */
 	    ubik_singleServer = 1;
 	if (nServers == 1 && !amIClone) {
-	    ubik_amSyncSite = 1;	/* let's start as sync site */
-	    syncSiteUntil = 0x7fffffff;	/* and be it quite a while */
+	    beacon_globals.ubik_amSyncSite = 1;	/* let's start as sync site */
+	    beacon_globals.syncSiteUntil = 0x7fffffff;	/* and be it quite a while */
 	}
     } else {
 	if (nServers == 1)	/* special case 1 server */
@@ -323,33 +377,36 @@ ubeacon_InitServerListCommon(afs_int32 ame, struct afsconf_cell *info,
     }
 
     if (ubik_singleServer) {
-	if (!ubik_amSyncSite)
+	if (!beacon_globals.ubik_amSyncSite)
 	    ubik_dprint("Ubik: I am the sync site - 1 server\n");
-	ubik_amSyncSite = 1;
-	syncSiteUntil = 0x7fffffff;	/* quite a while */
+	beacon_globals.ubik_amSyncSite = 1;
+	beacon_globals.syncSiteUntil = 0x7fffffff;	/* quite a while */
     }
     return 0;
 }
 
-/*! 
+/*!
  * \brief main lwp loop for code that sends out beacons.
- * 
+ *
  * This code only runs while we're sync site or we want to be the sync site.
  * It runs in its very own light-weight process.
  */
 void *
 ubeacon_Interact(void *dummy)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct timeval tt;
     struct rx_connection *connections[MAXSERVERS];
     struct ubik_server *servers[MAXSERVERS];
-    register afs_int32 i;
-    register struct ubik_server *ts;
+    afs_int32 i;
+    struct ubik_server *ts;
     afs_int32 temp, yesVotes, lastWakeupTime, oldestYesVote, syncsite;
     struct ubik_tid ttid;
+    struct ubik_version tversion;
     afs_int32 startTime;
 
+    afs_pthread_setname_self("beacon");
+
     /* loop forever getting votes */
     lastWakeupTime = 0;		/* keep track of time we last started a vote collection */
     while (1) {
@@ -384,19 +441,24 @@ ubeacon_Interact(void *dummy)
 	 * is a task for the recovery module, not the beacon module), and
 	 * prepare to send them an r multi-call containing the beacon message */
 	i = 0;			/* collect connections */
+	UBIK_BEACON_LOCK;
+	UBIK_ADDR_LOCK;
 	for (ts = ubik_servers; ts; ts = ts->next) {
 	    if (ts->up && ts->addr[0] != ubik_host[0]) {
 		servers[i] = ts;
 		connections[i++] = ts->vote_rxcid;
 	    }
 	}
+	UBIK_ADDR_UNLOCK;
+	UBIK_BEACON_UNLOCK;
 	servers[i] = (struct ubik_server *)0;	/* end of list */
-	/* note that we assume in the vote module that we'll always get at least BIGTIME 
+	/* note that we assume in the vote module that we'll always get at least BIGTIME
 	 * seconds of vote from anyone who votes for us, which means we can conservatively
 	 * assume we'll be fine until SMALLTIME seconds after we start collecting votes */
 	/* this next is essentially an expansion of rgen's ServBeacon routine */
 
-	ttid.epoch = ubik_epochTime;
+	UBIK_VERSION_LOCK;
+	ttid.epoch = version_globals.ubik_epochTime;
 	if (ubik_dbase->flags & DBWRITING) {
 	    /*
 	     * if a write is in progress, we have to send the writeTidCounter
@@ -407,55 +469,72 @@ ubeacon_Interact(void *dummy)
 	    ttid.counter = ubik_dbase->writeTidCounter;
 	} else
 	    ttid.counter = ubik_dbase->tidCounter + 1;
-#if defined(UBIK_PAUSE)
-	ubik_dbase->flags |= DBVOTING;
-#endif /* UBIK_PAUSE */
+	tversion.epoch = ubik_dbase->version.epoch;
+	tversion.counter = ubik_dbase->version.counter;
+	UBIK_VERSION_UNLOCK;
 
 	/* now analyze return codes, counting up our votes */
 	yesVotes = 0;		/* count how many to ensure we have quorum */
 	oldestYesVote = 0x7fffffff;	/* time quorum expires */
-	syncsite = ubeacon_AmSyncSite();
+	syncsite = amSyncSite();
+	if (!syncsite) {
+	    /* Ok to use the DB lock here since we aren't sync site */
+	    DBHOLD(ubik_dbase);
+	    urecovery_ResetState();
+	    DBRELE(ubik_dbase);
+	}
 	startTime = FT_ApproxTime();
 	/*
 	 * Don't waste time using mult Rx calls if there are no connections out there
 	 */
 	if (i > 0) {
+	    char hoststr[16];
 	    multi_Rx(connections, i) {
-		multi_VOTE_Beacon(syncsite, startTime, &ubik_dbase->version,
+		multi_VOTE_Beacon(syncsite, startTime, &tversion,
 				  &ttid);
 		temp = FT_ApproxTime();	/* now, more or less */
 		ts = servers[multi_i];
+		UBIK_BEACON_LOCK;
 		ts->lastBeaconSent = temp;
 		code = multi_error;
 		/* note that the vote time (the return code) represents the time
 		 * the vote was computed, *not* the time the vote expires.  We compute
 		 * the latter down below if we got enough votes to go with */
 		if (code > 0) {
-		    ts->lastVoteTime = code;
-		    if (code < oldestYesVote)
-			oldestYesVote = code;
-		    ts->lastVote = 1;
-		    if (!ts->isClone)
-			yesVotes += 2;
-		    if (ts->magic)
-			yesVotes++;	/* the extra epsilon */
-		    ts->up = 1;	/* server is up (not really necessary: recovery does this for real) */
-		    ts->beaconSinceDown = 1;
-		    ubik_dprint("yes vote from host %s\n",
-				afs_inet_ntoa(ts->addr[0]));
+		    if ((code & ~0xff) == ERROR_TABLE_BASE_RXK) {
+			ubik_dprint("token error %d from host %s\n",
+				    code, afs_inet_ntoa_r(ts->addr[0], hoststr));
+			ts->up = 0;
+			ts->beaconSinceDown = 0;
+			urecovery_LostServer(ts);
+		    } else {
+			ts->lastVoteTime = code;
+			if (code < oldestYesVote)
+			    oldestYesVote = code;
+			ts->lastVote = 1;
+			if (!ts->isClone)
+			    yesVotes += 2;
+			if (ts->magic)
+			    yesVotes++;	/* the extra epsilon */
+			ts->up = 1;	/* server is up (not really necessary: recovery does this for real) */
+			ts->beaconSinceDown = 1;
+			ubik_dprint("yes vote from host %s\n",
+				    afs_inet_ntoa_r(ts->addr[0], hoststr));
+		    }
 		} else if (code == 0) {
 		    ts->lastVoteTime = temp;
 		    ts->lastVote = 0;
 		    ts->beaconSinceDown = 1;
 		    ubik_dprint("no vote from %s\n",
-				afs_inet_ntoa(ts->addr[0]));
+				afs_inet_ntoa_r(ts->addr[0], hoststr));
 		} else if (code < 0) {
 		    ts->up = 0;
 		    ts->beaconSinceDown = 0;
-		    urecovery_LostServer();
+		    urecovery_LostServer(ts);
 		    ubik_dprint("time out from %s\n",
-				afs_inet_ntoa(ts->addr[0]));
+				afs_inet_ntoa_r(ts->addr[0], hoststr));
 		}
+		UBIK_BEACON_UNLOCK;
 	    }
 	    multi_End;
 	}
@@ -463,7 +542,7 @@ ubeacon_Interact(void *dummy)
 	 * the same restrictions apply for our voting for ourself as for our voting
 	 * for anyone else. */
 	i = SVOTE_Beacon((struct rx_call *)0, ubeacon_AmSyncSite(), startTime,
-			 &ubik_dbase->version, &ttid);
+			 &tversion, &ttid);
 	if (i) {
 	    yesVotes += 2;
 	    if (amIMagic)
@@ -471,27 +550,30 @@ ubeacon_Interact(void *dummy)
 	    if (i < oldestYesVote)
 		oldestYesVote = i;
 	}
-#if defined(UBIK_PAUSE)
-	ubik_dbase->flags &= ~DBVOTING;
-#endif /* UBIK_PAUSE */
 
 	/* now decide if we have enough votes to become sync site.
 	 * Note that we can still get enough votes even if we didn't for ourself. */
 	if (yesVotes > nServers) {	/* yesVotes is bumped by 2 or 3 for each site */
-	    if (!ubik_amSyncSite)
+	    UBIK_BEACON_LOCK;
+	    if (!beacon_globals.ubik_amSyncSite)
 		ubik_dprint("Ubik: I am the sync site\n");
-	    ubik_amSyncSite = 1;
-	    syncSiteUntil = oldestYesVote + SMALLTIME;
+	    beacon_globals.ubik_amSyncSite = 1;
+	    beacon_globals.syncSiteUntil = oldestYesVote + SMALLTIME;
 #ifndef AFS_PTHREAD_ENV
 		/* I did not find a corresponding LWP_WaitProcess(&ubik_amSyncSite) --
 		   this may be a spurious signal call -- sjenkins */
-		LWP_NoYieldSignal(&ubik_amSyncSite);
+		LWP_NoYieldSignal(&beacon_globals.ubik_amSyncSite);
 #endif
+	    UBIK_BEACON_UNLOCK;
 	} else {
-	    if (ubik_amSyncSite)
+	    UBIK_BEACON_LOCK;
+	    if (beacon_globals.ubik_amSyncSite)
 		ubik_dprint("Ubik: I am no longer the sync site\n");
-	    ubik_amSyncSite = 0;
+	    beacon_globals.ubik_amSyncSite = 0;
+	    UBIK_BEACON_UNLOCK;
+	    DBHOLD(ubik_dbase);
 	    urecovery_ResetState();	/* tell recovery we're no longer the sync site */
+	    DBRELE(ubik_dbase);
 	}
 
     }				/* while loop */
@@ -502,15 +584,15 @@ ubeacon_Interact(void *dummy)
  * \brief Verify that a given IP addresses does actually exist on this machine.
  *
  * \param ame      the pointer to my IP address specified in the
- *                 CellServDB file. 
- * \param aservers an array containing IP 
- *                 addresses of remote ubik servers. The array is 
+ *                 CellServDB file.
+ * \param aservers an array containing IP
+ *                 addresses of remote ubik servers. The array is
  *                 terminated by a zero address.
  *
  * Algorithm     : Verify that my IP addresses \p ame does actually exist
- *                 on this machine.  If any of my IP addresses are there 
- *                 in the remote server list \p aserver, remove them from 
- *                 this list.  Update global variable \p ubik_host[] with 
+ *                 on this machine.  If any of my IP addresses are there
+ *                 in the remote server list \p aserver, remove them from
+ *                 this list.  Update global variable \p ubik_host[] with
  *                 my IP addresses.
  *
  * \return 0 on success, non-zero on failure
@@ -520,6 +602,7 @@ verifyInterfaceAddress(afs_uint32 *ame, struct afsconf_cell *info,
 		       afs_uint32 aservers[]) {
     afs_uint32 myAddr[UBIK_MAX_INTERFACE_ADDR], *servList, tmpAddr;
     afs_uint32 myAddr2[UBIK_MAX_INTERFACE_ADDR];
+    char hoststr[16];
     int tcount, count, found, i, j, totalServers, start, end, usednetfiles =
 	0;
 
@@ -533,14 +616,14 @@ verifyInterfaceAddress(afs_uint32 *ame, struct afsconf_cell *info,
     if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || AFSDIR_SERVER_NETINFO_FILEPATH) {
 	/*
 	 * Find addresses we are supposed to register as per the netrestrict file
-	 * if it exists, else just register all the addresses we find on this 
+	 * if it exists, else just register all the addresses we find on this
 	 * host as returned by rx_getAllAddr (in NBO)
 	 */
 	char reason[1024];
-	count =
-	    parseNetFiles(myAddr, NULL, NULL, UBIK_MAX_INTERFACE_ADDR, reason,
-			  AFSDIR_SERVER_NETINFO_FILEPATH,
-			  AFSDIR_SERVER_NETRESTRICT_FILEPATH);
+	count = afsconf_ParseNetFiles(myAddr, NULL, NULL,
+				      UBIK_MAX_INTERFACE_ADDR, reason,
+				      AFSDIR_SERVER_NETINFO_FILEPATH,
+				      AFSDIR_SERVER_NETRESTRICT_FILEPATH);
 	if (count < 0) {
 	    ubik_print("ubik: Can't register any valid addresses:%s\n",
 		       reason);
@@ -568,7 +651,7 @@ verifyInterfaceAddress(afs_uint32 *ame, struct afsconf_cell *info,
 
     if (!found) {
 	ubik_print("ubik: primary address %s does not exist\n",
-		   afs_inet_ntoa(*ame));
+		   afs_inet_ntoa_r(*ame, hoststr));
 	/* if we had the result of rx_getAllAddr already, avoid subverting
 	 * the "is gethostbyname(gethostname()) us" check. If we're
 	 * using NetInfo/NetRestrict, we assume they have enough clue
@@ -595,7 +678,7 @@ verifyInterfaceAddress(afs_uint32 *ame, struct afsconf_cell *info,
     }
 
     /* if any of my addresses are there in serverList, then
-     ** use that as my primary addresses : the higher level 
+     ** use that as my primary addresses : the higher level
      ** application screwed up in dealing with multihomed concepts
      */
     for (j = 0, found = 0; j < count; j++) {
@@ -613,11 +696,11 @@ verifyInterfaceAddress(afs_uint32 *ame, struct afsconf_cell *info,
 	}
     }
     if (found)
-	ubik_print("Using %s as my primary address\n", afs_inet_ntoa(*ame));
+	ubik_print("Using %s as my primary address\n", afs_inet_ntoa_r(*ame, hoststr));
 
     if (!info) {
-	/* get rid of servers which were purged because all 
-	 ** those interface addresses are myself 
+	/* get rid of servers which were purged because all
+	 ** those interface addresses are myself
 	 */
 	for (start = 0, end = totalServers - 1; (start < end); start++, end--) {
 	    /* find the first zero entry from the beginning */
@@ -636,8 +719,8 @@ verifyInterfaceAddress(afs_uint32 *ame, struct afsconf_cell *info,
 	}
     }
 
-    /* update all my addresses in ubik_host in such a way 
-     ** that ubik_host[0] has the primary address 
+    /* update all my addresses in ubik_host in such a way
+     ** that ubik_host[0] has the primary address
      */
     ubik_host[0] = *ame;
     for (j = 0, i = 1; j < count; j++)
@@ -648,30 +731,33 @@ verifyInterfaceAddress(afs_uint32 *ame, struct afsconf_cell *info,
 }
 
 
-/*! 
+/*!
  * \brief Exchange IP address information with remote servers.
  *
  * \param ubik_host an array containing all my IP addresses.
  *
- * Algorithm     : Do an RPC to all remote ubik servers infroming them 
+ * Algorithm     : Do an RPC to all remote ubik servers informing them
  *                 about my IP addresses. Get their IP addresses and
  *                 update my linked list of ubik servers \p ubik_servers
  *
  * \return 0 on success, non-zero on failure
  */
 int
-updateUbikNetworkAddress(afs_uint32 ubik_host[UBIK_MAX_INTERFACE_ADDR])
+ubeacon_updateUbikNetworkAddress(afs_uint32 ubik_host[UBIK_MAX_INTERFACE_ADDR])
 {
     int j, count, code = 0;
     UbikInterfaceAddr inAddr, outAddr;
     struct rx_connection *conns[MAXSERVERS];
     struct ubik_server *ts, *server[MAXSERVERS];
     char buffer[32];
+    char hoststr[16];
 
+    UBIK_ADDR_LOCK;
     for (count = 0, ts = ubik_servers; ts; count++, ts = ts->next) {
 	conns[count] = ts->disk_rxcid;
 	server[count] = ts;
     }
+    UBIK_ADDR_UNLOCK;
 
 
     /* inform all other servers only if there are more than one
@@ -688,30 +774,51 @@ updateUbikNetworkAddress(afs_uint32 ubik_host[UBIK_MAX_INTERFACE_ADDR])
 	    multi_DISK_UpdateInterfaceAddr(&inAddr, &outAddr);
 	    ts = server[multi_i];	/* reply received from this server */
 	    if (!multi_error) {
+		UBIK_ADDR_LOCK;
 		if (ts->addr[0] != htonl(outAddr.hostAddr[0])) {
 		    code = UBADHOST;
-		    strcpy(buffer, (char *)afs_inet_ntoa(ts->addr[0]));
+		    strcpy(buffer, afs_inet_ntoa_r(ts->addr[0], hoststr));
 		    ubik_print("ubik:Two primary addresses for same server \
-                    %s %s\n", buffer, afs_inet_ntoa(htonl(outAddr.hostAddr[0])));
+                    %s %s\n", buffer,
+		    afs_inet_ntoa_r(htonl(outAddr.hostAddr[0]), hoststr));
 		} else {
 		    for (j = 1; j < UBIK_MAX_INTERFACE_ADDR; j++)
 			ts->addr[j] = htonl(outAddr.hostAddr[j]);
 		}
+		UBIK_ADDR_UNLOCK;
 	    } else if (multi_error == RXGEN_OPCODE) {	/* pre 3.5 remote server */
+		UBIK_ADDR_LOCK;
 		ubik_print
 		    ("ubik server %s does not support UpdateInterfaceAddr RPC\n",
-		     afs_inet_ntoa(ts->addr[0]));
+		     afs_inet_ntoa_r(ts->addr[0], hoststr));
+		UBIK_ADDR_UNLOCK;
 	    } else if (multi_error == UBADHOST) {
 		code = UBADHOST;	/* remote CellServDB inconsistency */
 		ubik_print("Inconsistent Cell Info on server: ");
+		UBIK_ADDR_LOCK;
 		for (j = 0; j < UBIK_MAX_INTERFACE_ADDR && ts->addr[j]; j++)
-		    ubik_print("%s ", afs_inet_ntoa(ts->addr[j]));
+		    ubik_print("%s ", afs_inet_ntoa_r(ts->addr[j], hoststr));
+		UBIK_ADDR_UNLOCK;
 		ubik_print("\n");
 	    } else {
+		UBIK_BEACON_LOCK;
 		ts->up = 0;	/* mark the remote server as down */
+		UBIK_BEACON_UNLOCK;
 	    }
 	}
 	multi_End;
     }
     return code;
 }
+
+void
+ubik_SetClientSecurityProcs(int (*secproc) (void *,
+					    struct rx_securityClass **,
+					    afs_int32 *),
+			    int (*checkproc) (void *),
+			    void *rock)
+{
+    secLayerProc = secproc;
+    tokenCheckProc = checkproc;
+    securityRock = rock;
+}
