@@ -34,74 +34,34 @@
 #include <afs/param.h>
 #include <afs/stds.h>
 
-#include <stdio.h>
-#include <setjmp.h>
-#include <sys/types.h>
-#include <string.h>
-#include <ctype.h>
+#include <roken.h>
 
 #ifdef AFS_NT40_ENV
 #include <windows.h>
-#include <winsock2.h>
 #define _CRT_RAND_S
-#include <stdlib.h>
-#include <process.h>
-#include <fcntl.h>
-#include <io.h>
 #include <afs/smb_iocons.h>
 #include <afs/afsd.h>
 #include <afs/cm_ioctl.h>
 #include <afs/pioctl_nt.h>
 #include <WINNT/syscfg.h>
 #else
-#include <sys/param.h>
-#include <sys/file.h>
-#include <sys/ioctl.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <netdb.h>
-#include <string.h>
-#include <fcntl.h>
-#include <pwd.h>
-#include <afs/venus.h>
-#include <sys/time.h>
-#include <netdb.h>
 #include <afs/afsint.h>
 #define FSINT_COMMON_XG 1
 #endif
-#include <sys/stat.h>
-#include <errno.h>
-#include <signal.h>
-#include <afs/vice.h>
+
+#include <afs/opr.h>
 #include <afs/cmd.h>
 #include <afs/auth.h>
 #include <afs/vlserver.h>
 #include <afs/ihandle.h>
 #include <afs/com_err.h>
 #include <afs/afscp.h>
-#ifdef HAVE_DIRENT_H
-#include <dirent.h>
-#endif
+
 #ifdef HAVE_DIRECT_H
 #include <direct.h>
 #endif
-#include <afs/errors.h>
-#include <afs/sys_prototypes.h>
-#include <des_prototypes.h>
-#include <rx/rx_prototypes.h>
-#include "../rxkad/md5.h"
-#ifdef O_LARGEFILE
-#define afs_stat        stat64
-#define afs_fstat       fstat64
-#define afs_open        open64
-#else /* !O_LARGEFILE */
-#define afs_stat        stat
-#define afs_fstat       fstat
-#define afs_open        open
-#endif /* !O_LARGEFILE */
+#include <hcrypto/md5.h>
 #ifdef AFS_PTHREAD_ENV
-#include <assert.h>
 pthread_key_t uclient_key;
 #endif
 
@@ -115,10 +75,12 @@ static int CmdProlog(struct cmd_syndesc *, char **, char **,
 static int ScanFid(char *, struct AFSFid *);
 static afs_int32 GetVenusFidByFid(char *, char *, int, struct afscp_venusfid **);
 static afs_int32 GetVenusFidByPath(char *, char *, struct afscp_venusfid **);
-static int BreakUpPath(char *, char **, char **);
+static int BreakUpPath(char *, char *, char *, size_t);
 
 static char pnp[AFSPATHMAX];	/* filename of this program when called */
 static int verbose = 0;		/* Set if -verbose option given */
+static int clear = 0;		/* Set if -clear option given,
+				   Unset if -crypt given; default is -crypt */
 static int cellGiven = 0;	/* Set if -cell option given */
 static int force = 0;		/* Set if -force option given */
 static int readlock = 0;	/* Set if -readlock option given */
@@ -225,23 +187,6 @@ summarizeMD5(char *fname)
 	    htonl(md5int[1]), htonl(md5int[2]), htonl(md5int[3]), p);
 } /* summarizeMD5 */
 
-#ifdef AFS_NT40_ENV
-static void
-ConvertAFSPath(char **fnp)
-{
-    char *p;
-
-    for (p = *fnp; *p; p++) {
-        if (*p == '\\')
-           *p = '/';
-    }
-
-    p = *fnp;
-    if (p[0] == '/' && p[1] == '/')
-        *fnp = p+1;
-}
-#endif /* AFS_NT40_ENV */
-
 /*!
  * parses all command-line arguments
  *
@@ -278,20 +223,20 @@ CmdProlog(struct cmd_syndesc *as, char **cellp, char **realmp,
 	if (pdp->items != NULL) {
 	    if (strcmp(pdp->name, "-verbose") == 0)
 	        verbose = 1;
+	    if (strcmp(pdp->name, "-clear") == 0)
+	        clear = 1;
+	    if (strcmp(pdp->name, "-crypt") == 0)
+	        clear = 0;
             else if (strcmp(pdp->name, "-md5") == 0)
 		md5sum = 1;	/* global */
             else if (strcmp(pdp->name, "-cell") == 0) {
 		cellGiven = 1;	/* global */
 		*cellp = pdp->items->data;
-            } else if ( strcmp(pdp->name, "-file") == 0) {
+            } else if ( (strcmp(pdp->name, "-file") == 0) ||
+                        (strcmp(pdp->name, "-fid") == 0) ||
+                        (strcmp(pdp->name, "-vnode") == 0) )
 		*fnp = pdp->items->data;
-#ifdef AFS_NT40_ENV
-                ConvertAFSPath(fnp);
-#endif /* AFS_NT40_ENV */
-            } else if ( (strcmp(pdp->name, "-fid") == 0) ||
-                        (strcmp(pdp->name, "-vnode") == 0) ) {
-		*fnp = pdp->items->data;
-            } else if (strcmp(pdp->name, "-force") == 0)
+            else if (strcmp(pdp->name, "-force") == 0)
 		force = 1;	/* global */
             else if (strcmp(pdp->name, "-synthesize") == 0)
 		*slp = pdp->items->data;
@@ -310,26 +255,16 @@ int
 main(int argc, char **argv)
 {
     struct cmd_syndesc *ts;
-    char *baseName;
-    int code;
+    char baseName[AFSNAMEMAX];
 
     /* try to get only the base name of this executable for use in logs */
-#ifdef AFS_NT40_ENV
-    char *p = strdup(argv[0]);
-    ConvertAFSPath(&p);
-    code = BreakUpPath(p, NULL, &baseName);
-    free(p);
-#else
-    code = BreakUpPath(argv[0], NULL, &baseName);
-#endif
-    if (code > 0)
+    if (BreakUpPath(argv[0], NULL, baseName, AFSNAMEMAX) > 0)
 	strlcpy(pnp, baseName, AFSNAMEMAX);
     else
 	strlcpy(pnp, argv[0], AFSPATHMAX);
-    free(baseName);
 
 #ifdef AFS_PTHREAD_ENV
-    assert(pthread_key_create(&uclient_key, NULL) == 0);
+    opr_Verify(pthread_key_create(&uclient_key, NULL) == 0);
 #endif
 
     ts = cmd_CreateSyntax("lock", lockFile, (void *)LockWrite,
@@ -337,6 +272,8 @@ main(int argc, char **argv)
     cmd_AddParm(ts, "-file", CMD_SINGLE, CMD_REQUIRED, "AFS-filename");
     cmd_AddParm(ts, "-cell", CMD_SINGLE, CMD_OPTIONAL, "cellname");
     cmd_AddParm(ts, "-verbose", CMD_FLAG, CMD_OPTIONAL, (char *)0);
+    cmd_AddParm(ts, "-clear", CMD_FLAG, CMD_OPTIONAL, (char *)0);
+    cmd_AddParm(ts, "-crypt", CMD_FLAG, CMD_OPTIONAL, (char *)0);
     cmd_Seek(ts, 4);
     cmd_AddParm(ts, "-realm", CMD_SINGLE, CMD_OPTIONAL, "REALMNAME");
     cmd_AddParm(ts, "-waitseconds", CMD_SINGLE, CMD_OPTIONAL, "seconds to wait before giving up");
@@ -349,6 +286,8 @@ main(int argc, char **argv)
 		"volume.vnode.uniquifier");
     cmd_AddParm(ts, "-cell", CMD_SINGLE, CMD_OPTIONAL, "cellname");
     cmd_AddParm(ts, "-verbose", CMD_FLAG, CMD_OPTIONAL, (char *)0);
+    cmd_AddParm(ts, "-clear", CMD_FLAG, CMD_OPTIONAL, (char *)0);
+    cmd_AddParm(ts, "-crypt", CMD_FLAG, CMD_OPTIONAL, (char *)0);
     cmd_Seek(ts, 4);
     cmd_AddParm(ts, "-realm", CMD_SINGLE, CMD_OPTIONAL, "REALMNAME");
     cmd_AddParm(ts, "-waitseconds", CMD_SINGLE, CMD_OPTIONAL, "seconds to wait before giving up");
@@ -359,6 +298,8 @@ main(int argc, char **argv)
     cmd_AddParm(ts, "-file", CMD_SINGLE, CMD_REQUIRED, "AFS-filename");
     cmd_AddParm(ts, "-cell", CMD_SINGLE, CMD_OPTIONAL, "cellname");
     cmd_AddParm(ts, "-verbose", CMD_FLAG, CMD_OPTIONAL, (char *)0);
+    cmd_AddParm(ts, "-clear", CMD_FLAG, CMD_OPTIONAL, (char *)0);
+    cmd_AddParm(ts, "-crypt", CMD_FLAG, CMD_OPTIONAL, (char *)0);
     cmd_Seek(ts, 4);
     cmd_AddParm(ts, "-realm", CMD_SINGLE, CMD_OPTIONAL, "REALMNAME");
     cmd_AddParm(ts, "-waitseconds", CMD_SINGLE, CMD_OPTIONAL, "seconds to wait before giving up");
@@ -370,6 +311,8 @@ main(int argc, char **argv)
 		"volume.vnode.uniquifier");
     cmd_AddParm(ts, "-cell", CMD_SINGLE, CMD_OPTIONAL, "cellname");
     cmd_AddParm(ts, "-verbose", CMD_FLAG, CMD_OPTIONAL, (char *)0);
+    cmd_AddParm(ts, "-clear", CMD_FLAG, CMD_OPTIONAL, (char *)0);
+    cmd_AddParm(ts, "-crypt", CMD_FLAG, CMD_OPTIONAL, (char *)0);
     cmd_Seek(ts, 4);
     cmd_AddParm(ts, "-realm", CMD_SINGLE, CMD_OPTIONAL, "REALMNAME");
     cmd_AddParm(ts, "-waitseconds", CMD_SINGLE, CMD_OPTIONAL, "seconds to wait before giving up");
@@ -379,6 +322,8 @@ main(int argc, char **argv)
     cmd_AddParm(ts, "-file", CMD_SINGLE, CMD_REQUIRED, "AFS-filename");
     cmd_AddParm(ts, "-cell", CMD_SINGLE, CMD_OPTIONAL, "cellname");
     cmd_AddParm(ts, "-verbose", CMD_FLAG, CMD_OPTIONAL, (char *)0);
+    cmd_AddParm(ts, "-clear", CMD_FLAG, CMD_OPTIONAL, (char *)0);
+    cmd_AddParm(ts, "-crypt", CMD_FLAG, CMD_OPTIONAL, (char *)0);
     cmd_AddParm(ts, "-md5", CMD_FLAG, CMD_OPTIONAL, "calculate md5 checksum");
     cmd_AddParm(ts, "-realm", CMD_SINGLE, CMD_OPTIONAL, "REALMNAME");
 
@@ -389,6 +334,7 @@ main(int argc, char **argv)
 		"volume.vnode.uniquifier");
     cmd_AddParm(ts, "-cell", CMD_SINGLE, CMD_OPTIONAL, "cellname");
     cmd_AddParm(ts, "-verbose", CMD_FLAG, CMD_OPTIONAL, (char *)0);
+    cmd_AddParm(ts, "-clear", CMD_FLAG, CMD_OPTIONAL, (char *)0);
     cmd_AddParm(ts, "-md5", CMD_FLAG, CMD_OPTIONAL, "calculate md5 checksum");
     cmd_AddParm(ts, "-realm", CMD_SINGLE, CMD_OPTIONAL, "REALMNAME");
 
@@ -397,6 +343,8 @@ main(int argc, char **argv)
     cmd_AddParm(ts, "-file", CMD_SINGLE, CMD_REQUIRED, "AFS-filename");
     cmd_AddParm(ts, "-cell", CMD_SINGLE, CMD_OPTIONAL, "cellname");
     cmd_AddParm(ts, "-verbose", CMD_FLAG, CMD_OPTIONAL, (char *)0);
+    cmd_AddParm(ts, "-clear", CMD_FLAG, CMD_OPTIONAL, (char *)0);
+    cmd_AddParm(ts, "-crypt", CMD_FLAG, CMD_OPTIONAL, (char *)0);
     cmd_AddParm(ts, "-md5", CMD_FLAG, CMD_OPTIONAL, "calculate md5 checksum");
     cmd_AddParm(ts, "-force", CMD_FLAG, CMD_OPTIONAL,
 		"overwrite existing file");
@@ -411,6 +359,8 @@ main(int argc, char **argv)
 		"volume.vnode.uniquifier");
     cmd_AddParm(ts, "-cell", CMD_SINGLE, CMD_OPTIONAL, "cellname");
     cmd_AddParm(ts, "-verbose", CMD_FLAG, CMD_OPTIONAL, (char *)0);
+    cmd_AddParm(ts, "-clear", CMD_FLAG, CMD_OPTIONAL, (char *)0);
+    cmd_AddParm(ts, "-crypt", CMD_FLAG, CMD_OPTIONAL, (char *)0);
     cmd_AddParm(ts, "-md5", CMD_FLAG, CMD_OPTIONAL, "calculate md5 checksum");
     cmd_AddParm(ts, "-force", CMD_FLAG, CMD_OPTIONAL,
 		"overwrite existing file");
@@ -421,6 +371,8 @@ main(int argc, char **argv)
     cmd_AddParm(ts, "-file", CMD_SINGLE, CMD_REQUIRED, "AFS-filename");
     cmd_AddParm(ts, "-cell", CMD_SINGLE, CMD_OPTIONAL, "cellname");
     cmd_AddParm(ts, "-verbose", CMD_FLAG, CMD_OPTIONAL, (char *)0);
+    cmd_AddParm(ts, "-clear", CMD_FLAG, CMD_OPTIONAL, (char *)0);
+    cmd_AddParm(ts, "-crypt", CMD_FLAG, CMD_OPTIONAL, (char *)0);
     cmd_AddParm(ts, "-realm", CMD_SINGLE, CMD_OPTIONAL, "REALMNAME");
 
     ts = cmd_CreateSyntax("fidappend", writeFile, NULL,
@@ -430,6 +382,8 @@ main(int argc, char **argv)
 		"volume.vnode.uniquifier");
     cmd_AddParm(ts, "-cell", CMD_SINGLE, CMD_OPTIONAL, "cellname");
     cmd_AddParm(ts, "-verbose", CMD_FLAG, CMD_OPTIONAL, (char *)0);
+    cmd_AddParm(ts, "-clear", CMD_FLAG, CMD_OPTIONAL, (char *)0);
+    cmd_AddParm(ts, "-crypt", CMD_FLAG, CMD_OPTIONAL, (char *)0);
     cmd_AddParm(ts, "-realm", CMD_SINGLE, CMD_OPTIONAL, "REALMNAME");
 
     if (afscp_Init(NULL) != 0)
@@ -501,13 +455,12 @@ GetVenusFidByFid(char *fidString, char *cellName, int onlyRW,
     struct afscp_volume *avolp;
 
     if (*avfpp == NULL) {
-	*avfpp = malloc(sizeof(struct afscp_venusfid));
+	*avfpp = calloc(1, sizeof(struct afscp_venusfid));
 	if ( *avfpp == NULL ) {
 	    code = ENOMEM;
 	    return code;
 	}
     }
-    memset(*avfpp, 0, sizeof(struct afscp_venusfid));
 
     if (cellName == NULL) {
 	(*avfpp)->cell = afscp_DefaultCell();
@@ -565,25 +518,22 @@ GetVenusFidByFid(char *fidString, char *cellName, int onlyRW,
  * Split a full path up into dirName and baseName components
  *
  * \param[in]	fullPath	can be absolute, relative, or local
- * \param[out]	dirName		pointer to output string or NULL
- * \param[out]	baseName	pointer to output string or NULL
+ * \param[out]	dirName		pointer to allocated char buffer or NULL
+ * \param[out]	baseName	pointer to allocated char buffer or NULL
  *
- * \post A buffer of appropriate size will be allocated into the output
- *       parameter baseName and the rightmost full path component of the
- *       fullPath copied into it; likewise, the other components of the
- *       fullPath (minus the trailing path separator) will be placed into
- *       the dirName output, which is also allocated to be the appropriate
- *       size.  If either dirName or baseName are NULL, only the non-NULL
- *       pointer will be allocated and filled in (but both can't be null
- *       or it would be pointless) -- so the caller can retrieve, say,
- *       only baseName if desired.  The return code is the number of
- *       strings allocated and copied:
+ * \post To the fulleset extent possible, the rightmost full path
+ *       component will be copied into baseName and all other
+ *       components into dirName (minus the trailing path separator).
+ *       If either dirName or baseName are NULL, only the non-NULL
+ *       pointer will be filled in (but both can't be null or it would
+ *       be pointless) -- so the caller can retrieve, say, only baseName
+ *       if desired.  The return code is the number of strings copied:
  *       0 if neither dirName nor baseName could be filled in
  *       1 if either dirName or baseName were filled in
  *       2 if both dirName and baseName were filled in
  */
 static int
-BreakUpPath(char *fullPath, char **dirName, char **baseName)
+BreakUpPath(char *fullPath, char *dirName, char *baseName, size_t baseNameSize)
 {
     char *lastSlash;
     size_t dirNameLen = 0;
@@ -593,43 +543,36 @@ BreakUpPath(char *fullPath, char **dirName, char **baseName)
 	return code;
     }
 
-    /* Track what we need to output and initialize output variables to NULL. */
     if (dirName == NULL)
 	useDirName = 0;
-    else
-	*dirName = NULL;
     if (baseName == NULL)
 	useBaseName = 0;
-    else
-	*baseName = NULL;
     if (!useBaseName && !useDirName) {
 	/* would be pointless to continue -- must be error in call */
 	return code;
     }
+#ifdef AFS_NT40_ENV
+    lastSlash = strrchr(fullPath, '\\');
+#else
     lastSlash = strrchr(fullPath, '/');
+#endif
     if (lastSlash != NULL) {
 	/* then lastSlash points to the last path separator in fullPath */
 	if (useDirName) {
 	    dirNameLen = strlen(fullPath) - strlen(lastSlash);
-	    *dirName = strdup(fullPath);
-	    if (*dirName != NULL) {
-		code++;
-		/* Wastes some memory, but avoids needing libroken. */
-		*dirName[dirNameLen] = '\0';
-	    }
+	    strlcpy(dirName, fullPath, min(dirNameLen + 1, baseNameSize));
+	    code++;
 	}
 	if (useBaseName) {
 	    lastSlash++;
-	    *baseName = strdup(lastSlash);
-	    if (*baseName != NULL)
-		code++;
+	    strlcpy(baseName, lastSlash, min(strlen(lastSlash) + 1, baseNameSize));
+	    code++;
 	}
     } else {
 	/* there are no path separators in fullPath -- it's just a baseName */
 	if (useBaseName) {
-	    *baseName = strdup(fullPath);
-	    if (*baseName != NULL)
-		code++;
+	    strlcpy(baseName, fullPath, min(strlen(fullPath) + 1, baseNameSize));
+	    code++;
 	}
     }
     return code;
@@ -698,6 +641,8 @@ lockFile(struct cmd_syndesc *as, void *arock)
 
     CmdProlog(as, &cell, &realm, &fname, NULL);
     afscp_AnonymousAuth(1);
+    if (clear)
+	afscp_Insecure();
 
     if ((locktype == LockWrite) && readlock)
 	locktype = LockRead;
@@ -784,6 +729,8 @@ readFile(struct cmd_syndesc *as, void *unused)
 
     CmdProlog(as, &cell, &realm, &fname, NULL);
     afscp_AnonymousAuth(1);
+    if (clear)
+	afscp_Insecure();
 
     if (md5sum)
 	MD5_Init(&md5);
@@ -827,14 +774,13 @@ readFile(struct cmd_syndesc *as, void *unused)
     Len <<= 32;
     Len += OutStatus.Length;
     ZeroInt64(Pos);
-    buf = (char *) malloc(bufflen * sizeof(char));
+    buf = calloc(bufflen, sizeof(char));
     if (buf == NULL) {
 	code = ENOMEM;
 	afs_com_err(pnp, code, "(cannot allocate buffer)");
 	afscp_FreeFid(avfp);
 	return code;
     }
-    memset(buf, 0, bufflen * sizeof(char));
     length = Len;
     while (!code && NonZeroInt64(length)) {
 	if (length > bufflen)
@@ -886,12 +832,13 @@ writeFile(struct cmd_syndesc *as, void *unused)
     afs_int64 Pos;
     afs_int64 length, Len, synthlength = 0, offset = 0;
     afs_int64 bytes;
+    int worstCode = 0;
     int synthesize = 0;
     int overWrite = 0;
     struct wbuf *bufchain = 0;
     struct wbuf *previous, *tbuf;
-    char *dirName = NULL;
-    char *baseName = NULL;
+    char dirName[AFSPATHMAX];
+    char baseName[AFSNAMEMAX];
     char ipv4_addr[16];
 
 #ifdef AFS_NT40_ENV
@@ -901,6 +848,8 @@ writeFile(struct cmd_syndesc *as, void *unused)
 
     CmdProlog(as, &cell, &realm, &fname, &sSynthLen);
     afscp_AnonymousAuth(1);
+    if (clear)
+	afscp_Insecure();
 
     if (realm != NULL)
 	code = afscp_SetDefaultRealm(realm);
@@ -913,7 +862,7 @@ writeFile(struct cmd_syndesc *as, void *unused)
 	if (code != 0) {
 	    afs_com_err(pnp, code, "(invalid value for synthesize length %s)",
 			sSynthLen);
-	    goto cleanup;
+	    return code;
 	}
 	synthesize = 1;
     }
@@ -922,7 +871,7 @@ writeFile(struct cmd_syndesc *as, void *unused)
 	code = GetVenusFidByFid(fname, cell, 1, &newvfp);
 	if (code != 0) {
 	    afs_com_err(pnp, code, "(GetVenusFidByFid returned code %d)", code);
-	    goto cleanup;
+	    return code;
 	}
     } else {
 	code = GetVenusFidByPath(fname, cell, &newvfp);
@@ -935,21 +884,23 @@ writeFile(struct cmd_syndesc *as, void *unused)
 		 * appending to it unless user forces overwrite
 		 */
 		code = EEXIST;
+		afscp_FreeFid(newvfp);
 		afs_com_err(pnp, code, "(use -force to overwrite)");
-		goto cleanup;
+		return code;
 	    }
 	} else { /* file not found */
 	    if (append) {
 		code = ENOENT;
 		afs_com_err(pnp, code, "(cannot append to non-existent file)");
-		goto cleanup;
+		return code;
 	    }
 	}
 	if (!append && !overWrite) { /* must create a new file in this case */
-	    if ( BreakUpPath(fname, &dirName, &baseName) != 2 ) {
+	    if ( BreakUpPath(fname, dirName, baseName, AFSNAMEMAX) != 2 ) {
 		code = EINVAL;
 		afs_com_err(pnp, code, "(must provide full AFS path)");
-		goto cleanup;
+		afscp_FreeFid(newvfp);
+		return code;
 	    }
 
 	    code = GetVenusFidByPath(dirName, cell, &dirvfp);
@@ -957,7 +908,7 @@ writeFile(struct cmd_syndesc *as, void *unused)
 	    newvfp = NULL;
 	    if (code != 0) {
 		afs_com_err(pnp, code, "(is dir %s in AFS?)", dirName);
-		goto cleanup;
+		return code;
 	    }
 	}
     }
@@ -965,7 +916,9 @@ writeFile(struct cmd_syndesc *as, void *unused)
     if ( (newvfp != NULL) && (newvfp->fid.Vnode & 1) ) {
 	code = EISDIR;
 	afs_com_err(pnp, code, "(%s is a directory, not a file)", fname);
-	goto cleanup;
+	afscp_FreeFid(newvfp);
+	afscp_FreeFid(dirvfp);
+	return code;
     }
     gettimeofday(&starttime, &Timezone);
 
@@ -979,7 +932,7 @@ writeFile(struct cmd_syndesc *as, void *unused)
 		        baseName, afs_printable_uint32_lu(dirvfp->fid.Volume),
 		        afs_printable_uint32_lu(dirvfp->fid.Vnode),
 		        afs_printable_uint32_lu(dirvfp->fid.Unique));
-	    goto cleanup;
+	    return code;
 	}
     }
     code = afscp_GetStatus(newvfp, &OutStatus);
@@ -987,7 +940,9 @@ writeFile(struct cmd_syndesc *as, void *unused)
 	afs_inet_ntoa_r(newvfp->cell->fsservers[0]->addrs[0], ipv4_addr);
 	afs_com_err(pnp, code, "(failed to get status of file %s from"
 		    "server %s, code = %d)", fname, ipv4_addr, code);
-	goto cleanup;
+	afscp_FreeFid(newvfp);
+	afscp_FreeFid(dirvfp);
+	return code;
     }
 
     if ( !append && !force &&
@@ -998,8 +953,10 @@ writeFile(struct cmd_syndesc *as, void *unused)
 	 * (covers fidwrite edge case)
 	 */
 	code = EEXIST;
+	afscp_FreeFid(newvfp);
+	afscp_FreeFid(dirvfp);
 	afs_com_err(pnp, code, "(use -force to overwrite)");
-	goto cleanup;
+	return code;
     }
 
     if (append) {
@@ -1022,16 +979,17 @@ writeFile(struct cmd_syndesc *as, void *unused)
      */
     Len = 0;
     while (Len < WRITEBUFLEN) {
-	tbuf = (struct wbuf *)malloc(sizeof(struct wbuf));
+	tbuf = calloc(1, sizeof(struct wbuf));
 	if (tbuf == NULL) {
 	    if (!bufchain) {
 		code = ENOMEM;
+		afscp_FreeFid(newvfp);
+		afscp_FreeFid(dirvfp);
 		afs_com_err(pnp, code, "(cannot allocate buffer)");
-		goto cleanup;
+		return code;
 	    }
 	    break;
 	}
-	memset(tbuf, 0, sizeof(struct wbuf));
 	tbuf->buflen = BUFFLEN;
 	if (synthesize) {
 	    afs_int64 ll, l = tbuf->buflen;
@@ -1112,6 +1070,8 @@ writeFile(struct cmd_syndesc *as, void *unused)
 	    }
 	}
     }
+    afscp_FreeFid(newvfp);
+    afscp_FreeFid(dirvfp);
 
     gettimeofday(&writetime, &Timezone);
     if (code) {
@@ -1129,10 +1089,5 @@ writeFile(struct cmd_syndesc *as, void *unused)
     if (md5sum)
 	summarizeMD5(fname);
 
-cleanup:
-    free(baseName);
-    free(dirName);
-    afscp_FreeFid(newvfp);
-    afscp_FreeFid(dirvfp);
-    return code;
+    return worstCode;
 } /* writeFile */
