@@ -40,10 +40,11 @@
 
 #undef SHARED
 
-#include <rx/xdr.h>
+#include <afs/opr.h>
 #include <afs/nfs.h>
+#include <rx/rx_queue.h>
 #include <lwp.h>
-#include <lock.h>
+#include <opr/lock.h>
 #include <afs/cmd.h>
 #include <afs/ptclient.h>
 #include <afs/afsint.h>
@@ -68,14 +69,11 @@
 #include <afs/partition.h>
 #include <afs/dir.h>
 #ifndef AFS_NT40_ENV
-# include <afs/netutils.h>
+# include <afs/softsig.h>
 #endif
 #include "viced_prototypes.h"
 #include "viced.h"
 #include "host.h"
-#ifdef AFS_PTHREAD_ENV
-# include <afs/softsig.h>
-#endif
 #if defined(AFS_SGI_ENV)
 # include "sys/schedctl.h"
 # include "sys/lock.h"
@@ -96,12 +94,8 @@ static afs_int32 Do_VLRegisterRPC(void);
 int eventlog = 0, rxlog = 0;
 FILE *debugFile;
 
-#ifdef AFS_PTHREAD_ENV
 pthread_mutex_t fsync_glock_mutex;
 pthread_cond_t fsync_cond;
-#else
-char fsync_wait[1];
-#endif /* AFS_PTHREAD_ENV */
 
 #ifdef AFS_NT40_ENV
 # define NT_OPEN_MAX    1024	/* This is an arbitrary no. we came up with for
@@ -177,9 +171,7 @@ static int offline_shutdown_timeout = -1; /* -offline-shutdown-timeout option */
 
 struct timeval tp;
 
-#ifdef AFS_PTHREAD_ENV
 pthread_key_t viced_uclient_key;
-#endif
 
 /*
  * FileServer's name and IP address, both network byte order and
@@ -188,6 +180,7 @@ pthread_key_t viced_uclient_key;
 #define ADDRSPERSITE 16		/* Same global is in rx/rx_user.c */
 
 char FS_HostName[128] = "localhost";
+char *FS_configPath = NULL;
 afs_uint32 FS_HostAddr_NBO;
 afs_uint32 FS_HostAddr_HBO;
 afs_uint32 FS_HostAddrs[ADDRSPERSITE], FS_HostAddr_cnt = 0, FS_registered = 0;
@@ -235,8 +228,8 @@ static int fs_stateInit(void)
     fs_state.options.fs_state_verify_before_save = 1;
     fs_state.options.fs_state_verify_after_restore = 1;
 
-    CV_INIT(&fs_state.worker_done_cv, "worker done", CV_DEFAULT, 0);
-    osi_Assert(pthread_rwlock_init(&fs_state.state_lock, NULL) == 0);
+    opr_cv_init(&fs_state.worker_done_cv, "worker done");
+    opr_Verify(pthread_rwlock_init(&fs_state.state_lock, NULL) == 0);
 }
 # endif /* AFS_NT40_ENV */
 #endif /* AFS_DEMAND_ATTACH_FS */
@@ -269,7 +262,6 @@ CheckDescriptors(void *unused)
 }				/*CheckDescriptors */
 
 
-#ifdef AFS_PTHREAD_ENV
 void
 CheckSignal_Signal(int x)
 {
@@ -287,33 +279,33 @@ CheckDescriptors_Signal(int x)
 {
     CheckDescriptors(NULL);
 }
-#else /* AFS_PTHREAD_ENV */
-void
-CheckSignal_Signal(int x)
-{
-    IOMGR_SoftSig(CheckSignal, 0);
-}
 
-void
-ShutDown_Signal(int x)
-{
-    IOMGR_SoftSig(ShutDown, 0);
-}
-
-void
-CheckDescriptors_Signal(int x)
-{
-    IOMGR_SoftSig(CheckDescriptors, 0);
-}
-#endif /* AFS_PTHREAD_ENV */
-
-/* check whether caller is authorized to manage RX statistics */
+/* check whether caller is authorized to perform admin operations */
 int
-fs_rxstat_userok(struct rx_call *call)
+viced_SuperUser(struct rx_call *call)
 {
     return afsconf_SuperUser(confDir, call, NULL);
 }
 
+/**
+ * Return true if this name is a member of the local realm.
+ */
+int
+fs_IsLocalRealmMatch(void *rock, char *name, char *inst, char *cell)
+{
+    struct afsconf_dir *dir = (struct afsconf_dir *)rock;
+    afs_int32 islocal = 0;	/* default to no */
+    int code;
+
+    code = afsconf_IsLocalRealmMatch(dir, &islocal, name, inst, cell);
+    if (code) {
+	ViceLog(0,
+		("Failed local realm check; code=%d, name=%s, inst=%s, cell=%s\n",
+		 code, name, inst, cell));
+    }
+    return islocal;
+}
+
 static void
 ResetCheckSignal(void)
 {
@@ -327,7 +319,7 @@ ResetCheckSignal(void)
     signo = SIGXCPU;
 #endif
 
-#if defined(AFS_PTHREAD_ENV) && !defined(AFS_NT40_ENV)
+#if !defined(AFS_NT40_ENV)
     softsig_signal(signo, CheckSignal_Signal);
 #else
     signal(signo, CheckSignal_Signal);
@@ -338,11 +330,7 @@ static void
 ResetCheckDescriptors(void)
 {
 #ifndef AFS_NT40_ENV
-# if defined(AFS_PTHREAD_ENV)
     softsig_signal(SIGTERM, CheckDescriptors_Signal);
-# else
-    (void)signal(SIGTERM, CheckDescriptors_Signal);
-# endif
 #endif
 }
 
@@ -395,7 +383,7 @@ CheckAdminName(void)
 static void
 setThreadId(char *s)
 {
-#if defined(AFS_PTHREAD_ENV) && !defined(AFS_NT40_ENV)
+#if !defined(AFS_NT40_ENV)
     int threadId;
 
     /* set our 'thread-id' so that the host hold table works */
@@ -424,11 +412,7 @@ FiveMinuteCheckLWP(void *unused)
     while (1) {
 #endif
 
-#ifdef AFS_PTHREAD_ENV
 	sleep(fiveminutes);
-#else /* AFS_PTHREAD_ENV */
-	IOMGR_Sleep(fiveminutes);
-#endif /* AFS_PTHREAD_ENV */
 
 #ifdef AFS_DEMAND_ATTACH_FS
 	FS_STATE_WRLOCK;
@@ -449,11 +433,8 @@ FiveMinuteCheckLWP(void *unused)
 	if (FS_registered == 1)
 	    Do_VLRegisterRPC();
 	/* Force wakeup in case we missed something; pthreads does timedwait */
-#ifndef AFS_PTHREAD_ENV
-	LWP_NoYieldSignal(fsync_wait);
-#endif
 	if (printBanner && (++msg & 1)) {	/* Every 10 minutes */
-	    time_t now = FT_ApproxTime();
+	    time_t now = time(NULL);
 	    struct tm tm;
 	    strftime(tbuffer, sizeof(tbuffer), "%a %b %d %H:%M:%S %Y",
 		     localtime_r(&now, &tm));
@@ -467,7 +448,7 @@ FiveMinuteCheckLWP(void *unused)
 #ifdef AFS_DEMAND_ATTACH_FS
     fs_state.FiveMinuteLWP_tranquil = 1;
     FS_LOCK;
-    CV_BROADCAST(&fs_state.worker_done_cv);
+    opr_cv_broadcast(&fs_state.worker_done_cv);
     FS_UNLOCK;
     FS_STATE_UNLOCK;
 #endif
@@ -494,11 +475,7 @@ HostCheckLWP(void *unused)
     while(1) {
 #endif
 
-#ifdef AFS_PTHREAD_ENV
 	sleep(fiveminutes);
-#else /* AFS_PTHREAD_ENV */
-	IOMGR_Sleep(fiveminutes);
-#endif /* AFS_PTHREAD_ENV */
 
 #ifdef AFS_DEMAND_ATTACH_FS
 	FS_STATE_WRLOCK;
@@ -519,7 +496,7 @@ HostCheckLWP(void *unused)
 #ifdef AFS_DEMAND_ATTACH_FS
     fs_state.HostCheckLWP_tranquil = 1;
     FS_LOCK;
-    CV_BROADCAST(&fs_state.worker_done_cv);
+    opr_cv_broadcast(&fs_state.worker_done_cv);
     FS_UNLOCK;
     FS_STATE_UNLOCK;
 #endif
@@ -534,9 +511,7 @@ static void *
 FsyncCheckLWP(void *unused)
 {
     afs_int32 code;
-#ifdef AFS_PTHREAD_ENV
     struct timespec fsync_next;
-#endif
     ViceLog(1, ("Starting fsync check process\n"));
 
     setThreadId("FsyncCheckLWP");
@@ -550,19 +525,14 @@ FsyncCheckLWP(void *unused)
     while(1) {
 #endif
 	FSYNC_LOCK;
-#ifdef AFS_PTHREAD_ENV
 	/* rounding is fine */
 	fsync_next.tv_nsec = 0;
 	fsync_next.tv_sec = time(0) + fiveminutes;
 
-	code = CV_TIMEDWAIT(&fsync_cond, &fsync_glock_mutex,
+	code = opr_cv_timedwait(&fsync_cond, &fsync_glock_mutex,
 			    &fsync_next);
 	if (code != 0 && code != ETIMEDOUT)
 	    ViceLog(0, ("pthread_cond_timedwait returned %d\n", code));
-#else /* AFS_PTHREAD_ENV */
-	if ((code = LWP_WaitProcess(fsync_wait)) != LWP_SUCCESS)
-	    ViceLog(0, ("LWP_WaitProcess returned %d\n", code));
-#endif /* AFS_PTHREAD_ENV */
 	FSYNC_UNLOCK;
 
 #ifdef AFS_DEMAND_ATTACH_FS
@@ -585,7 +555,7 @@ FsyncCheckLWP(void *unused)
 #ifdef AFS_DEMAND_ATTACH_FS
     fs_state.FsyncCheckLWP_tranquil = 1;
     FS_LOCK;
-    CV_BROADCAST(&fs_state.worker_done_cv);
+    opr_cv_broadcast(&fs_state.worker_done_cv);
     FS_UNLOCK;
     FS_STATE_UNLOCK;
 #endif /* AFS_DEMAND_ATTACH_FS */
@@ -624,7 +594,6 @@ ClearXStatValues(void)
      * Zero all xstat-related structures.
      */
     memset((&afs_perfstats), 0, sizeof(struct afs_PerfStats));
-#if FS_STATS_DETAILED
     memset((&afs_FullPerfStats), 0,
 	   sizeof(struct fs_stats_FullPerfStats));
 
@@ -656,7 +625,6 @@ ClearXStatValues(void)
 #  endif
     afs_perfstats.sysname_ID = SYS_NAME_ID_UNDEFINED;
 # endif /* SYS_NAME_ID */
-#endif
 
 }				/*ClearXStatValues */
 
@@ -676,7 +644,7 @@ PrintCounters(void)
     int stats_flags = 0;
 #endif
 
-    FT_GetTimeOfDay(&tpl, 0);
+    gettimeofday(&tpl, 0);
     Statistics = 1;
     strftime(tbuffer, sizeof(tbuffer), "%a %b %d %H:%M:%S %Y",
 	     localtime_r(&StartTime, &tm));
@@ -759,17 +727,11 @@ ShutDownAndCore(int dopanic)
     char tbuffer[32];
 
     if (dopanic) {
-#ifdef AFS_PTHREAD_ENV
 	pthread_t watchdogPid;
 	pthread_attr_t tattr;
-	osi_Assert(pthread_attr_init(&tattr) == 0);
-	osi_Assert(pthread_create(&watchdogPid, &tattr, ShutdownWatchdogLWP, NULL) == 0);
-#else
-	PROCESS watchdogPid;
-	osi_Assert(LWP_CreateProcess
-	       (ShutdownWatchdogLWP, stack * 1024, LWP_MAX_PRIORITY - 2,
-	        NULL, "ShutdownWatchdog", &watchdogPid) == LWP_SUCCESS);
-#endif
+	opr_Verify(pthread_attr_init(&tattr) == 0);
+	opr_Verify(pthread_create(&watchdogPid, &tattr,
+				  ShutdownWatchdogLWP, NULL) == 0);
     }
 
     /* do not allows new reqests to be served from now on, all new requests
@@ -824,7 +786,7 @@ ShutDownAndCore(int dopanic)
 		FS_LOCK;
 		FS_STATE_UNLOCK;
 		ViceLog(0, ("waiting for background host/callback threads to quiesce before saving fileserver state...\n"));
-		CV_WAIT(&fs_state.worker_done_cv, &fileproc_glock_mutex);
+		opr_cv_wait(&fs_state.worker_done_cv, &fileproc_glock_mutex);
 		FS_UNLOCK;
 		FS_STATE_RDLOCK;
 	    }
@@ -945,13 +907,13 @@ ParseRights(char *arights)
 static int
 max_fileserver_thread(void)
 {
-#if defined(AFS_PTHREAD_ENV) && (defined(AFS_AIX_ENV) || defined(AFS_HPUX_ENV))
+#if defined(AFS_AIX_ENV) || defined(AFS_HPUX_ENV)
     long ans;
 
     ans = sysconf(_SC_THREAD_THREADS_MAX);
     if (0 < ans && ans < MAX_FILESERVER_THREAD)
 	return (int)ans;
-#endif /* defined(AFS_PTHREAD_ENV) */
+#endif
     return MAX_FILESERVER_THREAD;
 }
 
@@ -1024,7 +986,7 @@ ParseArgs(int argc, char *argv[])
 {
     int code;
     int optval;
-    char *optstring;
+    char *optstring = NULL;
     struct cmd_item *optlist;
     struct cmd_syndesc *opts;
 
@@ -1083,10 +1045,8 @@ ParseArgs(int argc, char *argv[])
     cmd_AddParmAtOffset(opts, OPT_hostcpsrefresh, "-hr", CMD_SINGLE,
 			CMD_OPTIONAL, "hours between host CPS refreshes");
 
-#if defined(AFS_PTHREAD_ENV)
     cmd_AddParmAtOffset(opts, OPT_vattachthreads, "-vattachpar", CMD_SINGLE,
 			CMD_OPTIONAL, "# of volume attachment threads");
-#endif
 
     cmd_AddParmAtOffset(opts, OPT_abortthreshold, "-abortthreshold",
 			CMD_SINGLE, CMD_OPTIONAL,
@@ -1201,6 +1161,11 @@ ParseArgs(int argc, char *argv[])
     if (code)
 	return -1;
 
+    /* XXX - cmd_OptionAsString(opts, OPT_config, &configDir); */
+
+    cmd_OpenConfigFile(AFSDIR_SERVER_CONFIG_FILE_FILEPATH);
+    cmd_SetCommandName("fileserver");
+
     if (cmd_OptionPresent(opts, OPT_large)
 	&& cmd_OptionPresent(opts, OPT_small)) {
 	printf("Only one of -L or -S must be specified\n");
@@ -1291,9 +1256,7 @@ ParseArgs(int argc, char *argv[])
 	hostaclRefresh = optval * 60 * 60;
     }
 
-#ifdef AFS_PTHREAD_ENV
     cmd_OptionAsInt(opts, OPT_vattachthreads, &vol_attach_threads);
-#endif /* AFS_PTHREAD_ENV */
 
     cmd_OptionAsInt(opts, OPT_abortthreshold, &abort_threshold);
 
@@ -1302,13 +1265,6 @@ ParseArgs(int argc, char *argv[])
 	busyonrst = 0;
 
     if (cmd_OptionAsInt(opts, OPT_offline_timeout, &offline_timeout) == 0) {
-#ifndef AFS_PTHREAD_ENV
-	if (offline_timeout != -1) {
-	    printf("The only valid -offline-timeout value for the LWP "
-		   "fileserver is -1\n");
-	    return -1;
-	}
-#endif /* AFS_PTHREAD_ENV */
 	if (offline_timeout < -1) {
 	    printf("Invalid -offline-timeout value %d; the only valid "
 		   "negative value is -1\n", offline_timeout);
@@ -1318,13 +1274,6 @@ ParseArgs(int argc, char *argv[])
 
     if (cmd_OptionAsInt(opts, OPT_offline_shutdown_timeout,
 			&offline_shutdown_timeout) == 0) {
-#ifndef AFS_PTHREAD_ENV
-	if (offline_shutdown_timeout != -1) {
-	    printf("The only valid -offline-shutdown-timeout value for the "
-		   "LWP fileserver is -1\n");
-	    return -1;
-	}
-#endif /* AFS_PTHREAD_ENV */
 	if (offline_shutdown_timeout < -1) {
 	    printf("Invalid -offline-timeout value %d; the only valid "
 		   "negative value is -1\n", offline_shutdown_timeout);
@@ -1393,7 +1342,6 @@ ParseArgs(int argc, char *argv[])
 	optstring = NULL;
     }
 
-    /* XXX - cmd_OptionAsString(opts, OPT_config, &configDir); */
     cmd_OptionAsInt(opts, OPT_debug, &LogLevel);
     /* XXX - cmd_OptionAsString(opts, OPT_logfile, &logFile); */
     cmd_OptionAsFlag(opts, OPT_mrafslogs, &mrafsStyleLogs);
@@ -1426,15 +1374,7 @@ ParseArgs(int argc, char *argv[])
     cmd_OptionAsFlag(opts, OPT_rxdbge, &eventlog);
     cmd_OptionAsInt(opts, OPT_rxpck, &rxpackets);
 
-    if (cmd_OptionAsInt(opts, OPT_rxmaxmtu, &rxMaxMTU) == 0) {
-	if ((rxMaxMTU < RX_MIN_PACKET_SIZE) ||
-	    (rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
-	    printf("rxMaxMTU %d invalid; must be between %d-%" AFS_SIZET_FMT "\n",
-		   rxMaxMTU, RX_MIN_PACKET_SIZE,
-		   RX_MAX_PACKET_DATA_SIZE);
-	    return(-1);
-	}
-    }
+    cmd_OptionAsInt(opts, OPT_rxmaxmtu, &rxMaxMTU);
 
     if (cmd_OptionAsInt(opts, OPT_udpsize, &optval) == 0) {
 	if (optval < rx_GetMinUdpBufSize()) {
@@ -1447,8 +1387,6 @@ ParseArgs(int argc, char *argv[])
     /* rxkad options */
     cmd_OptionAsFlag(opts, OPT_dotted, &rxkadDisableDotCheck);
     if (cmd_OptionAsList(opts, OPT_realm, &optlist) == 0) {
-	extern char local_realms[AFS_NUM_LREALMS][AFS_REALM_SZ];
-	extern int num_lrealms;
 
 	for (; optlist != NULL; optlist=optlist->next) {
 	    if (strlen(optlist->data) >= AFS_REALM_SZ) {
@@ -1456,17 +1394,7 @@ ParseArgs(int argc, char *argv[])
 		       "characters.\n", AFS_REALM_SZ);
 		return -1;
 	    }
-
-	    if (num_lrealms == -1)
-		num_lrealms = 0;
-	    if (num_lrealms >= AFS_NUM_LREALMS) {
-		printf("a maximum of %d -realm arguments can be "
-		       "specified.\n", AFS_NUM_LREALMS);
-	        return -1;
-	    }
-
-	    strncpy(local_realms[num_lrealms++], optlist->data, AFS_REALM_SZ);
-	    optlist = optlist->next;
+	    afsconf_SetLocalRealm(optlist->data); /* overrides krb.conf file, if one */
 	}
     }
 
@@ -1530,9 +1458,7 @@ InitPR(void)
 	return code;
     }
 
-#ifdef AFS_PTHREAD_ENV
-    osi_Assert(pthread_key_create(&viced_uclient_key, NULL) == 0);
-#endif
+    opr_Verify(pthread_key_create(&viced_uclient_key, NULL) == 0);
 
     SystemId = SYSADMINID;
     SystemAnyUser = ANYUSERID;
@@ -1558,38 +1484,28 @@ InitPR(void)
 	AnonymousID = ANONYMOUSID;
 	return 0;
       sleep:
-#ifdef AFS_PTHREAD_ENV
 	sleep(30);
-#else /* AFS_PTHREAD_ENV */
-	IOMGR_Sleep(30);
-#endif /* AFS_PTHREAD_ENV */
     }
 }				/*InitPR */
 
-struct rx_connection *serverconns[MAXSERVERS];
-struct ubik_client *cstruct;
+static struct ubik_client *cstruct;
 
-afs_int32
-vl_Initialize(const char *confDir)
+static afs_int32
+vl_Initialize(struct afsconf_dir *dir)
 {
     afs_int32 code, i;
     afs_int32 scIndex = RX_SECIDX_NULL;
-    struct afsconf_dir *tdir;
-    struct rx_securityClass *sc;
     struct afsconf_cell info;
+    struct rx_securityClass *sc;
+    struct rx_connection *serverconns[MAXSERVERS];
 
-    tdir = afsconf_Open(confDir);
-    if (!tdir) {
-	ViceLog(0,
-		("Could not open configuration directory (%s).\n", confDir));
-	exit(1);
-    }
-    code = afsconf_ClientAuth(tdir, &sc, &scIndex);
+    memset(serverconns, 0, sizeof(serverconns));
+    code = afsconf_ClientAuth(dir, &sc, &scIndex);
     if (code) {
 	ViceLog(0, ("Could not get security object for localAuth\n"));
 	exit(1);
     }
-    code = afsconf_GetCellInfo(tdir, NULL, AFSCONF_VLDBSERVICE, &info);
+    code = afsconf_GetCellInfo(dir, NULL, AFSCONF_VLDBSERVICE, &info);
     if (info.numServers > MAXSERVERS) {
 	ViceLog(0,
 		("vl_Initialize: info.numServers=%d (> MAXSERVERS=%d)\n",
@@ -1602,7 +1518,6 @@ vl_Initialize(const char *confDir)
 			     info.hostAddr[i].sin_port, USER_SERVICE_ID, sc,
 			     scIndex);
     code = ubik_ClientInit(serverconns, &cstruct);
-    afsconf_Close(tdir);
     if (code) {
 	ViceLog(0, ("vl_Initialize: ubik client init failed.\n"));
 	return code;
@@ -1704,8 +1619,8 @@ WriteSysIdFile(void)
 	/*
 	 * File exists; keep the old one around
 	 */
-	renamefile(AFSDIR_SERVER_SYSID_FILEPATH,
-		   AFSDIR_SERVER_OLDSYSID_FILEPATH);
+	rk_rename(AFSDIR_SERVER_SYSID_FILEPATH,
+		  AFSDIR_SERVER_OLDSYSID_FILEPATH);
     }
     fd = afs_open(AFSDIR_SERVER_SYSID_FILEPATH, O_WRONLY | O_TRUNC | O_CREAT,
 		  0666);
@@ -1811,10 +1726,12 @@ SetupVL(void)
 	 * /usr/afs/local/NetRestict)
 	 */
 	char reason[1024];
-	afs_int32 code = parseNetFiles(FS_HostAddrs, NULL, NULL,
-				       ADDRSPERSITE, reason,
-				       AFSDIR_SERVER_NETINFO_FILEPATH,
-				       AFSDIR_SERVER_NETRESTRICT_FILEPATH);
+	afs_int32 code;
+
+	code = afsconf_ParseNetFiles(FS_HostAddrs, NULL, NULL,
+				     ADDRSPERSITE, reason,
+				     AFSDIR_SERVER_NETINFO_FILEPATH,
+				     AFSDIR_SERVER_NETRESTRICT_FILEPATH);
 	if (code < 0) {
 	    ViceLog(0, ("Can't register any valid addresses: %s\n", reason));
 	    exit(1);
@@ -1833,7 +1750,7 @@ SetupVL(void)
 }
 
 afs_int32
-InitVL(void)
+InitVL(struct afsconf_dir *dir)
 {
     afs_int32 code;
 
@@ -1841,7 +1758,7 @@ InitVL(void)
      * If this fails, it's because something major is wrong, and is not
      * likely to be time dependent.
      */
-    code = vl_Initialize(AFSDIR_SERVER_ETC_DIRPATH);
+    code = vl_Initialize(dir);
     if (code != 0) {
 	ViceLog(0,
 		("Couldn't initialize volume location library; code=%d.\n", code));
@@ -1876,12 +1793,8 @@ main(int argc, char *argv[])
     struct rx_securityClass **securityClasses;
     afs_int32 numClasses;
     struct rx_service *tservice;
-#ifdef AFS_PTHREAD_ENV
     pthread_t serverPid;
     pthread_attr_t tattr;
-#else /* AFS_PTHREAD_ENV */
-    PROCESS parentPid, serverPid;
-#endif /* AFS_PTHREAD_ENV */
     struct hostent *he;
     int minVnodesRequired;	/* min size of vnode cache */
 #ifndef AFS_NT40_ENV
@@ -1922,9 +1835,7 @@ main(int argc, char *argv[])
     if (ParseArgs(argc, argv)) {
 	exit(-1);
     }
-#ifdef AFS_PTHREAD_ENV
-    MUTEX_INIT(&fileproc_glock_mutex, "fileproc", MUTEX_DEFAULT, 0);
-#endif /* AFS_PTHREAD_ENV */
+    opr_mutex_init(&fileproc_glock_mutex);
 
 #ifdef AFS_SGI_VNODE_GLUE
     if (afs_init_kernel_config(-1) < 0) {
@@ -1933,13 +1844,18 @@ main(int argc, char *argv[])
 	exit(1);
     }
 #endif
-    confDir = afsconf_Open(AFSDIR_SERVER_ETC_DIRPATH);
+    FS_configPath = strdup(AFSDIR_SERVER_ETC_DIRPATH);
+
+    confDir = afsconf_Open(FS_configPath);
     if (!confDir) {
 	fprintf(stderr, "Unable to open config directory %s\n",
-		AFSDIR_SERVER_ETC_DIRPATH);
+		FS_configPath);
 	exit(-1);
     }
 
+    /* initialize audit user check */
+    osi_audit_set_user_check(confDir, fs_IsLocalRealmMatch);
+
     /* Open FileLog on stdout, stderr, fd 1 and fd2 (for perror), sigh. */
 #ifndef AFS_NT40_ENV
     serverLogSyslogTag = "fileserver";
@@ -1949,7 +1865,7 @@ main(int argc, char *argv[])
 
     LogCommandLine(argc, argv, "starting", "", "File server", FSLog);
 
-#if defined(AFS_PTHREAD_ENV) && !defined(AFS_NT40_ENV)
+#if !defined(AFS_NT40_ENV)
     /* initialize the pthread soft signal handler thread */
     softsig_init();
 #endif
@@ -2018,19 +1934,13 @@ main(int argc, char *argv[])
 		("The system supports a max of %d open files and we are starting %d threads (ihandle fd cache is %d)\n",
 		 curLimit, lwps, vol_io_params.fd_max_cachesize));
     }
-#ifndef AFS_PTHREAD_ENV
-    osi_Assert(LWP_InitializeProcessSupport(LWP_MAX_PRIORITY - 2, &parentPid) ==
-	   LWP_SUCCESS);
-#endif /* !AFS_PTHREAD_ENV */
 
     /* Initialize volume support */
     if (!novbc) {
 	V_BreakVolumeCallbacks = BreakVolumeCallBacksLater;
     }
 
-#ifdef AFS_PTHREAD_ENV
     SetLogThreadNumProgram( rx_GetThreadNum );
-#endif
 
     /* initialize libacl routines */
     acl_Initialize(ACL_VERSION);
@@ -2069,7 +1979,10 @@ main(int argc, char *argv[])
 	rx_SetNoJumbo();
     }
     if (rxMaxMTU != -1) {
-	rx_SetMaxMTU(rxMaxMTU);
+	if (rx_SetMaxMTU(rxMaxMTU) != 0) {
+	    ViceLog(0, ("rxMaxMTU %d is invalid\n", rxMaxMTU));
+	    exit(1);
+	}
     }
     rx_GetIFInfo();
     rx_SetRxDeadTime(30);
@@ -2126,7 +2039,7 @@ main(int argc, char *argv[])
     InitCallBack(numberofcbs);
     ClearXStatValues();
 
-    code = InitVL();
+    code = InitVL(confDir);
     if (code && code != VL_MULTIPADDR) {
 	ViceLog(0, ("Fatal error in library initialization, exiting!!\n"));
 	exit(1);
@@ -2139,10 +2052,10 @@ main(int argc, char *argv[])
     }
 
     /* allow super users to manage RX statistics */
-    rx_SetRxStatUserOk(fs_rxstat_userok);
+    rx_SetRxStatUserOk(viced_SuperUser);
 
-    CV_INIT(&fsync_cond, "fsync", CV_DEFAULT, 0);
-    MUTEX_INIT(&fsync_glock_mutex, "fsync", MUTEX_DEFAULT, 0);
+    opr_cv_init(&fsync_cond);
+    opr_mutex_init(&fsync_glock_mutex);
 
 #if !defined(AFS_DEMAND_ATTACH_FS)
     /*
@@ -2209,7 +2122,7 @@ main(int argc, char *argv[])
     /* Install handler to catch the shutdown signal;
      * bosserver assumes SIGQUIT shutdown
      */
-#if defined(AFS_PTHREAD_ENV) && !defined(AFS_NT40_ENV)
+#if !defined(AFS_NT40_ENV)
     softsig_signal(SIGQUIT, ShutDown_Signal);
 #else
     (void)signal(SIGQUIT, ShutDown_Signal);
@@ -2238,34 +2151,19 @@ main(int argc, char *argv[])
      */
     ih_UseLargeCache();
 
-#ifdef AFS_PTHREAD_ENV
     ViceLog(5, ("Starting pthreads\n"));
-    osi_Assert(pthread_attr_init(&tattr) == 0);
-    osi_Assert(pthread_attr_setdetachstate(&tattr, PTHREAD_CREATE_DETACHED) == 0);
+    opr_Verify(pthread_attr_init(&tattr) == 0);
+    opr_Verify(pthread_attr_setdetachstate(&tattr,
+					   PTHREAD_CREATE_DETACHED) == 0);
 
-    osi_Assert(pthread_create
-	   (&serverPid, &tattr, FiveMinuteCheckLWP,
-	    &fiveminutes) == 0);
-    osi_Assert(pthread_create
-	   (&serverPid, &tattr, HostCheckLWP, &fiveminutes) == 0);
-    osi_Assert(pthread_create
-	   (&serverPid, &tattr, FsyncCheckLWP, &fiveminutes) == 0);
-#else /* AFS_PTHREAD_ENV */
-    ViceLog(5, ("Starting LWP\n"));
-    osi_Assert(LWP_CreateProcess
-	   (FiveMinuteCheckLWP, stack * 1024, LWP_MAX_PRIORITY - 2,
-	    (void *)&fiveminutes, "FiveMinuteChecks",
-	    &serverPid) == LWP_SUCCESS);
+    opr_Verify(pthread_create(&serverPid, &tattr, FiveMinuteCheckLWP,
+			      &fiveminutes) == 0);
+    opr_Verify(pthread_create(&serverPid, &tattr, HostCheckLWP,
+			      &fiveminutes) == 0);
+    opr_Verify(pthread_create(&serverPid, &tattr, FsyncCheckLWP,
+			      &fiveminutes) == 0);
 
-    osi_Assert(LWP_CreateProcess
-	   (HostCheckLWP, stack * 1024, LWP_MAX_PRIORITY - 2,
-	    (void *)&fiveminutes, "HostCheck", &serverPid) == LWP_SUCCESS);
-    osi_Assert(LWP_CreateProcess
-	   (FsyncCheckLWP, stack * 1024, LWP_MAX_PRIORITY - 2,
-	    (void *)&fiveminutes, "FsyncCheck", &serverPid) == LWP_SUCCESS);
-#endif /* AFS_PTHREAD_ENV */
-
-    FT_GetTimeOfDay(&tp, 0);
+    gettimeofday(&tp, 0);
 
     /*
      * Figure out the FileServer's name and primary address.
@@ -2295,15 +2193,9 @@ main(int argc, char *argv[])
     strftime(tbuffer, sizeof(tbuffer), "%a %b %d %H:%M:%S %Y",
 	     localtime_r(&t, &tm));
     ViceLog(0, ("File Server started %s\n", tbuffer));
-#if FS_STATS_DETAILED
     afs_FullPerfStats.det.epoch.tv_sec = StartTime = tp.tv_sec;
-#endif
-#ifdef AFS_PTHREAD_ENV
     while (1) {
 	sleep(1000);		/* long time */
     }
-#else /* AFS_PTHREAD_ENV */
-    osi_Assert(LWP_WaitProcess(&parentPid) == LWP_SUCCESS);
-#endif /* AFS_PTHREAD_ENV */
     return 0;
 }
