@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -20,12 +20,14 @@
 #define readwriteVolume		RWVOL
 #define readonlyVolume		ROVOL
 #define backupVolume		BACKVOL
+#define rwreplicaVolume		RWREPL
 
 #define RWVOL			0
 #define ROVOL			1
 #define BACKVOL			2
+#define RWREPL			3
 
-#define VOLMAXTYPES             3   /* _current_ max number of types */
+#define VOLMAXTYPES             4   /* _current_ max number of types */
 
 /* the maximum number of volumes in a volume group that we can handle */
 #define VOL_VG_MAX_VOLS 20
@@ -46,6 +48,7 @@
 #define VFORMAT "V%010lu.vol"
 #define	VHDREXT	".vol"
 #endif
+#define	VHDRNAMELEN (11 + sizeof(VHDREXT) - 1) /* must match VFORMAT */
 #define VMAXPATHLEN 64		/* Maximum length (including null) of a volume
 				 * external path name */
 
